@@ -647,6 +647,10 @@ rb_ec_setup_exception(const rb_execution_context_t *ec, VALUE mesg, VALUE cause)
         cause = get_ec_errinfo(ec);
     }
     if (cause != mesg) {
+        if (THROW_DATA_P(cause)) {
+            cause = Qnil;
+        }
+
         rb_ivar_set(mesg, id_cause, cause);
     }
 }
