@@ -3,9 +3,7 @@
 
 #include <ruby/ruby.h>
 #include "../digest.h"
-#if defined(SHA1_USE_OPENSSL)
-#include "sha1ossl.h"
-#elif defined(SHA1_USE_COMMONDIGEST)
+#if defined(SHA1_USE_COMMONDIGEST)
 #include "sha1cc.h"
 #else
 #include "sha1.h"
@@ -22,27 +20,44 @@ static const rb_digest_metadata_t sha1 = {
 };
 
 /*
+ * Document-class: Digest::SHA1 < Digest::Base
  * A class for calculating message digests using the SHA-1 Secure Hash
  * Algorithm by NIST (the US' National Institute of Standards and
  * Technology), described in FIPS PUB 180-1.
+ *
+ * See Digest::Instance for digest API.
+ *
+ * SHA-1 calculates a digest of 160 bits (20 bytes).
+ *
+ * == Examples
+ *  require 'digest'
+ *
+ *  # Compute a complete digest
+ *  Digest::SHA1.hexdigest 'abc'      #=> "a9993e36..."
+ *
+ *  # Compute digest by chunks
+ *  sha1 = Digest::SHA1.new               # =>#<Digest::SHA1>
+ *  sha1.update "ab"
+ *  sha1 << "c"                           # alias for #update
+ *  sha1.hexdigest                        # => "a9993e36..."
+ *
+ *  # Use the same object to compute another digest
+ *  sha1.reset
+ *  sha1 << "message"
+ *  sha1.hexdigest                        # => "6f9b9af3..."
  */
 void
 Init_sha1(void)
 {
     VALUE mDigest, cDigest_Base, cDigest_SHA1;
 
-    rb_require("digest");
-
 #if 0
     mDigest = rb_define_module("Digest"); /* let rdoc know */
 #endif
-    mDigest = rb_path2class("Digest");
+    mDigest = rb_digest_namespace();
     cDigest_Base = rb_path2class("Digest::Base");
 
     cDigest_SHA1 = rb_define_class_under(mDigest, "SHA1", cDigest_Base);
 
-#undef RUBY_UNTYPED_DATA_WARNING
-#define RUBY_UNTYPED_DATA_WARNING 0
-    rb_iv_set(cDigest_SHA1, "metadata",
-	      Data_Wrap_Struct(0, 0, 0, (void *)&sha1));
+    rb_iv_set(cDigest_SHA1, "metadata", rb_digest_make_metadata(&sha1));
 }
