@@ -13,10 +13,15 @@
 **********************************************************************/
 
 #include "gc.h"
+#include "id_table.h"
 #include "internal.h"
+#include "internal/array.h"
+#include "internal/class.h"
 #include "internal/hash.h"
 #include "internal/string.h"
 #include "internal/sanitizers.h"
+#include "symbol.h"
+#include "shape.h"
 #include "node.h"
 #include "objspace.h"
 #include "ruby/debug.h"
@@ -35,11 +40,13 @@ struct dump_config {
     const char *root_category;
     VALUE cur_obj;
     VALUE cur_obj_klass;
+    size_t cur_page_slot_size;
     size_t cur_obj_references;
     unsigned int roots: 1;
     unsigned int full_heap: 1;
     unsigned int partial_dump;
     size_t since;
+    size_t shapes_since;
     unsigned long buffer_len;
     char buffer[BUFFER_CAPACITY];
 };
@@ -75,7 +82,8 @@ buffer_ensure_capa(struct dump_config *dc, unsigned long requested)
     }
 }
 
-static void buffer_append(struct dump_config *dc, const char *cstr, unsigned long len)
+static void
+buffer_append(struct dump_config *dc, const char *cstr, unsigned long len)
 {
     if (LIKELY(len > 0)) {
         buffer_ensure_capa(dc, len);
@@ -141,10 +149,10 @@ dump_append_sizet(struct dump_config *dc, const size_t number)
 }
 
 static void
-dump_append_c(struct dump_config *dc, char c)
+dump_append_c(struct dump_config *dc, unsigned char c)
 {
     if (c <= 0x1f) {
-        const unsigned int width = (sizeof(c) * CHAR_BIT / 4) + 5;
+        const unsigned int width = rb_strlen_lit("\\u0000") + 1;
         buffer_ensure_capa(dc, width);
         unsigned long required = snprintf(dc->buffer + dc->buffer_len, width, "\\u00%02x", c);
         RUBY_ASSERT(required <= width);
@@ -162,7 +170,7 @@ dump_append_ref(struct dump_config *dc, VALUE ref)
 {
     RUBY_ASSERT(ref > 0);
 
-    char buffer[((sizeof(VALUE) * CHAR_BIT + 3) / 4) + 4];
+    char buffer[roomof(sizeof(VALUE) * CHAR_BIT, 4) + rb_strlen_lit("\"0x\"")];
     char *buffer_start, *buffer_end;
 
     buffer_start = buffer_end = &buffer[sizeof(buffer)];
@@ -234,32 +242,32 @@ obj_type(VALUE obj)
 {
     switch (BUILTIN_TYPE(obj)) {
 #define CASE_TYPE(type) case T_##type: return #type
-	CASE_TYPE(NONE);
-	CASE_TYPE(NIL);
-	CASE_TYPE(OBJECT);
-	CASE_TYPE(CLASS);
-	CASE_TYPE(ICLASS);
-	CASE_TYPE(MODULE);
-	CASE_TYPE(FLOAT);
-	CASE_TYPE(STRING);
-	CASE_TYPE(REGEXP);
-	CASE_TYPE(ARRAY);
-	CASE_TYPE(HASH);
-	CASE_TYPE(STRUCT);
-	CASE_TYPE(BIGNUM);
-	CASE_TYPE(FILE);
-	CASE_TYPE(FIXNUM);
-	CASE_TYPE(TRUE);
-	CASE_TYPE(FALSE);
-	CASE_TYPE(DATA);
-	CASE_TYPE(MATCH);
-	CASE_TYPE(SYMBOL);
-	CASE_TYPE(RATIONAL);
-	CASE_TYPE(COMPLEX);
-	CASE_TYPE(IMEMO);
-	CASE_TYPE(UNDEF);
-	CASE_TYPE(NODE);
-	CASE_TYPE(ZOMBIE);
+        CASE_TYPE(NONE);
+        CASE_TYPE(NIL);
+        CASE_TYPE(OBJECT);
+        CASE_TYPE(CLASS);
+        CASE_TYPE(ICLASS);
+        CASE_TYPE(MODULE);
+        CASE_TYPE(FLOAT);
+        CASE_TYPE(STRING);
+        CASE_TYPE(REGEXP);
+        CASE_TYPE(ARRAY);
+        CASE_TYPE(HASH);
+        CASE_TYPE(STRUCT);
+        CASE_TYPE(BIGNUM);
+        CASE_TYPE(FILE);
+        CASE_TYPE(FIXNUM);
+        CASE_TYPE(TRUE);
+        CASE_TYPE(FALSE);
+        CASE_TYPE(DATA);
+        CASE_TYPE(MATCH);
+        CASE_TYPE(SYMBOL);
+        CASE_TYPE(RATIONAL);
+        CASE_TYPE(COMPLEX);
+        CASE_TYPE(IMEMO);
+        CASE_TYPE(UNDEF);
+        CASE_TYPE(NODE);
+        CASE_TYPE(ZOMBIE);
 #undef CASE_TYPE
       default: break;
     }
@@ -312,6 +320,17 @@ reachable_object_i(VALUE ref, void *data)
     dc->cur_obj_references++;
 }
 
+static bool
+dump_string_ascii_only(const char *str, long size)
+{
+    for (long i = 0; i < size; i++) {
+        if (str[i] & 0x80) {
+            return false;
+        }
+    }
+    return true;
+}
+
 static void
 dump_append_string_content(struct dump_config *dc, VALUE obj)
 {
@@ -322,12 +341,34 @@ dump_append_string_content(struct dump_config *dc, VALUE obj)
         dump_append_sizet(dc, rb_str_capacity(obj));
     }
 
-    if (is_ascii_string(obj)) {
-        dump_append(dc, ", \"value\":");
-        dump_append_string_value(dc, obj);
+    if (RSTRING_LEN(obj) && rb_enc_asciicompat(rb_enc_from_index(ENCODING_GET(obj)))) {
+        int cr = ENC_CODERANGE(obj);
+        if (cr == RUBY_ENC_CODERANGE_UNKNOWN) {
+            if (dump_string_ascii_only(RSTRING_PTR(obj), RSTRING_LEN(obj))) {
+                cr = RUBY_ENC_CODERANGE_7BIT;
+            }
+        }
+        if (cr == RUBY_ENC_CODERANGE_7BIT) {
+            dump_append(dc, ", \"value\":");
+            dump_append_string_value(dc, obj);
+        }
     }
 }
 
+static inline void
+dump_append_id(struct dump_config *dc, ID id)
+{
+    if (is_instance_id(id)) {
+        dump_append_string_value(dc, rb_sym2str(ID2SYM(id)));
+    }
+    else {
+        dump_append(dc, "\"ID_INTERNAL(");
+        dump_append_sizet(dc, rb_id_to_serial(id));
+        dump_append(dc, ")\"");
+    }
+}
+
+
 static void
 dump_object(VALUE obj, struct dump_config *dc)
 {
@@ -344,7 +385,11 @@ dump_object(VALUE obj, struct dump_config *dc)
 
     dc->cur_obj = obj;
     dc->cur_obj_references = 0;
-    dc->cur_obj_klass = BUILTIN_TYPE(obj) == T_NODE ? 0 : RBASIC_CLASS(obj);
+    if (BUILTIN_TYPE(obj) == T_NODE || BUILTIN_TYPE(obj) == T_IMEMO) {
+        dc->cur_obj_klass = 0;
+    } else {
+        dc->cur_obj_klass = RBASIC_CLASS(obj);
+    }
 
     if (dc->partial_dump && (!ainfo || ainfo->generation < dc->since)) {
         return;
@@ -360,6 +405,13 @@ dump_object(VALUE obj, struct dump_config *dc)
     dump_append(dc, obj_type(obj));
     dump_append(dc, "\"");
 
+    size_t shape_id = rb_shape_get_shape_id(obj);
+    dump_append(dc, ", \"shape_id\":");
+    dump_append_sizet(dc, shape_id);
+
+    dump_append(dc, ", \"slot_size\":");
+    dump_append_sizet(dc, dc->cur_page_slot_size);
+
     if (dc->cur_obj_klass) {
         dump_append(dc, ", \"class\":");
         dump_append_ref(dc, dc->cur_obj_klass);
@@ -385,8 +437,6 @@ dump_object(VALUE obj, struct dump_config *dc)
       case T_STRING:
         if (STR_EMBED_P(obj))
             dump_append(dc, ", \"embedded\":true");
-        if (is_broken_string(obj))
-            dump_append(dc, ", \"broken\":true");
         if (FL_TEST(obj, RSTRING_FSTR))
             dump_append(dc, ", \"fstring\":true");
         if (STR_SHARED_P(obj))
@@ -399,6 +449,27 @@ dump_object(VALUE obj, struct dump_config *dc)
             dump_append(dc, rb_enc_name(rb_enc_from_index(ENCODING_GET(obj))));
             dump_append(dc, "\"");
         }
+
+        dump_append(dc, ", \"coderange\":\"");
+        switch (RB_ENC_CODERANGE(obj)) {
+          case RUBY_ENC_CODERANGE_UNKNOWN:
+            dump_append(dc, "unknown");
+            break;
+          case RUBY_ENC_CODERANGE_7BIT:
+            dump_append(dc, "7bit");
+            break;
+          case RUBY_ENC_CODERANGE_VALID:
+            dump_append(dc, "valid");
+            break;
+          case RUBY_ENC_CODERANGE_BROKEN:
+            dump_append(dc, "broken");
+            break;
+        }
+        dump_append(dc, "\"");
+
+        if (RB_ENC_CODERANGE(obj) == RUBY_ENC_CODERANGE_BROKEN)
+            dump_append(dc, ", \"broken\":true");
+
         break;
 
       case T_HASH:
@@ -413,9 +484,9 @@ dump_object(VALUE obj, struct dump_config *dc)
       case T_ARRAY:
         dump_append(dc, ", \"length\":");
         dump_append_ld(dc, RARRAY_LEN(obj));
-        if (RARRAY_LEN(obj) > 0 && FL_TEST(obj, ELTS_SHARED))
+        if (RARRAY_LEN(obj) > 0 && FL_TEST(obj, RARRAY_SHARED_FLAG))
             dump_append(dc, ", \"shared\":true");
-        if (RARRAY_LEN(obj) > 0 && FL_TEST(obj, RARRAY_EMBED_FLAG))
+        if (FL_TEST(obj, RARRAY_EMBED_FLAG))
             dump_append(dc, ", \"embedded\":true");
         break;
 
@@ -427,6 +498,9 @@ dump_object(VALUE obj, struct dump_config *dc)
         break;
 
       case T_CLASS:
+        dump_append(dc, ", \"variation_count\":");
+        dump_append_d(dc, RCLASS_EXT(obj)->variation_count);
+
       case T_MODULE:
         if (rb_class_get_superclass(obj)) {
             dump_append(dc, ", \"superclass\":");
@@ -439,7 +513,8 @@ dump_object(VALUE obj, struct dump_config *dc)
                 dump_append(dc, ", \"name\":\"");
                 dump_append(dc, RSTRING_PTR(mod_name));
                 dump_append(dc, "\"");
-            } else {
+            }
+            else {
                 VALUE real_mod_name = rb_mod_name(rb_class_real(obj));
                 if (RTEST(real_mod_name)) {
                     dump_append(dc, ", \"real_class_name\":\"");
@@ -470,7 +545,10 @@ dump_object(VALUE obj, struct dump_config *dc)
 
       case T_OBJECT:
         dump_append(dc, ", \"ivars\":");
-        dump_append_lu(dc, ROBJECT_NUMIV(obj));
+        dump_append_lu(dc, ROBJECT_IV_COUNT(obj));
+        if (rb_shape_obj_too_complex(obj)) {
+            dump_append(dc, ", \"too_complex_shape\":true");
+        }
         break;
 
       case T_FILE:
@@ -482,8 +560,8 @@ dump_object(VALUE obj, struct dump_config *dc)
         break;
 
       case T_ZOMBIE:
-          dump_append(dc, "}\n");
-          return;
+        dump_append(dc, "}\n");
+        return;
 
       default:
         break;
@@ -539,9 +617,10 @@ heap_i(void *vstart, void *vend, size_t stride, void *data)
     for (; v != (VALUE)vend; v += stride) {
         void *ptr = asan_poisoned_object_p(v);
         asan_unpoison_object(v, false);
+        dc->cur_page_slot_size = stride;
 
-	if (dc->full_heap || RBASIC(v)->flags)
-	    dump_object(v, dc);
+        if (dc->full_heap || RBASIC(v)->flags)
+            dump_object(v, dc);
 
         if (ptr) {
             asan_poison_object(v);
@@ -573,7 +652,7 @@ root_obj_i(const char *category, VALUE obj, void *data)
 }
 
 static void
-dump_output(struct dump_config *dc, VALUE output, VALUE full, VALUE since)
+dump_output(struct dump_config *dc, VALUE output, VALUE full, VALUE since, VALUE shapes)
 {
 
     dc->full_heap = 0;
@@ -582,7 +661,8 @@ dump_output(struct dump_config *dc, VALUE output, VALUE full, VALUE since)
     if (TYPE(output) == T_STRING) {
         dc->stream = Qfalse;
         dc->string = output;
-    } else {
+    }
+    else {
         dc->stream = output;
         dc->string = Qfalse;
     }
@@ -594,9 +674,12 @@ dump_output(struct dump_config *dc, VALUE output, VALUE full, VALUE since)
     if (RTEST(since)) {
         dc->partial_dump = 1;
         dc->since = NUM2SIZET(since);
-    } else {
+    }
+    else {
         dc->partial_dump = 0;
     }
+
+    dc->shapes_since = RTEST(shapes) ? NUM2SIZET(shapes) : 0;
 }
 
 static VALUE
@@ -606,28 +689,103 @@ dump_result(struct dump_config *dc)
 
     if (dc->string) {
         return dc->string;
-    } else {
+    }
+    else {
         rb_io_flush(dc->stream);
         return dc->stream;
     }
 }
 
+/* :nodoc: */
 static VALUE
 objspace_dump(VALUE os, VALUE obj, VALUE output)
 {
     struct dump_config dc = {0,};
-    dump_output(&dc, output, Qnil, Qnil);
+    if (!RB_SPECIAL_CONST_P(obj)) {
+        dc.cur_page_slot_size = rb_gc_obj_slot_size(obj);
+    }
+
+    dump_output(&dc, output, Qnil, Qnil, Qnil);
 
     dump_object(obj, &dc);
 
     return dump_result(&dc);
 }
 
+static void
+shape_i(rb_shape_t *shape, void *data)
+{
+    struct dump_config *dc = (struct dump_config *)data;
+
+    size_t shape_id = rb_shape_id(shape);
+    if (shape_id < dc->shapes_since) {
+        return;
+    }
+
+    dump_append(dc, "{\"address\":");
+    dump_append_ref(dc, (VALUE)shape);
+
+    dump_append(dc, ", \"type\":\"SHAPE\", \"id\":");
+    dump_append_sizet(dc, shape_id);
+
+    if (shape->type != SHAPE_ROOT) {
+        dump_append(dc, ", \"parent_id\":");
+        dump_append_lu(dc, shape->parent_id);
+    }
+
+    dump_append(dc, ", \"depth\":");
+    dump_append_sizet(dc, rb_shape_depth(shape));
+
+    dump_append(dc, ", \"shape_type\":");
+    switch((enum shape_type)shape->type) {
+      case SHAPE_ROOT:
+        dump_append(dc, "\"ROOT\"");
+        break;
+      case SHAPE_IVAR:
+        dump_append(dc, "\"IVAR\"");
+
+        dump_append(dc, ",\"edge_name\":");
+        dump_append_id(dc, shape->edge_name);
+
+        break;
+      case SHAPE_FROZEN:
+        dump_append(dc, "\"FROZEN\"");
+        break;
+      case SHAPE_CAPACITY_CHANGE:
+        dump_append(dc, "\"CAPACITY_CHANGE\"");
+        dump_append(dc, ", \"capacity\":");
+        dump_append_sizet(dc, shape->capacity);
+        break;
+      case SHAPE_INITIAL_CAPACITY:
+        dump_append(dc, "\"INITIAL_CAPACITY\"");
+        dump_append(dc, ", \"capacity\":");
+        dump_append_sizet(dc, shape->capacity);
+        break;
+      case SHAPE_T_OBJECT:
+        dump_append(dc, "\"T_OBJECT\"");
+        break;
+      case SHAPE_OBJ_TOO_COMPLEX:
+        dump_append(dc, "\"OBJ_TOO_COMPLEX\"");
+        break;
+      default:
+        rb_bug("[objspace] unexpected shape type");
+    }
+
+    dump_append(dc, ", \"edges\":");
+    dump_append_sizet(dc, rb_shape_edges_count(shape));
+
+    dump_append(dc, ", \"memsize\":");
+    dump_append_sizet(dc, rb_shape_memsize(shape));
+
+    dump_append(dc, "}\n");
+}
+
+/* :nodoc: */
 static VALUE
-objspace_dump_all(VALUE os, VALUE output, VALUE full, VALUE since)
+objspace_dump_all(VALUE os, VALUE output, VALUE full, VALUE since, VALUE shapes)
 {
     struct dump_config dc = {0,};
-    dump_output(&dc, output, full, since);
+    dump_output(&dc, output, full, since, shapes);
 
     if (!dc.partial_dump || dc.since == 0) {
         /* dump roots */
@@ -635,12 +793,29 @@ objspace_dump_all(VALUE os, VALUE output, VALUE full, VALUE since)
         if (dc.roots) dump_append(&dc, "]}\n");
     }
 
+    if (RTEST(shapes)) {
+        rb_shape_each_shape(shape_i, &dc);
+    }
+
     /* dump all objects */
     rb_objspace_each_objects(heap_i, &dc);
 
     return dump_result(&dc);
 }
 
+/* :nodoc: */
+static VALUE
+objspace_dump_shapes(VALUE os, VALUE output, VALUE shapes)
+{
+    struct dump_config dc = {0,};
+    dump_output(&dc, output, Qfalse, Qnil, shapes);
+
+    if (RTEST(shapes)) {
+        rb_shape_each_shape(shape_i, &dc);
+    }
+    return dump_result(&dc);
+}
+
 void
 Init_objspace_dump(VALUE rb_mObjSpace)
 {
@@ -650,7 +825,8 @@ Init_objspace_dump(VALUE rb_mObjSpace)
 #endif
 
     rb_define_module_function(rb_mObjSpace, "_dump", objspace_dump, 2);
-    rb_define_module_function(rb_mObjSpace, "_dump_all", objspace_dump_all, 3);
+    rb_define_module_function(rb_mObjSpace, "_dump_all", objspace_dump_all, 4);
+    rb_define_module_function(rb_mObjSpace, "_dump_shapes", objspace_dump_shapes, 2);
 
     /* force create static IDs */
     rb_obj_gc_flags(rb_mObjSpace, 0, 0);
