@@ -23,13 +23,26 @@
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
-#if RUBY_DEVEL && !defined RUBY_DEBUG_ENV
+#if defined RUBY_DEVEL && !defined RUBY_DEBUG_ENV
 # define RUBY_DEBUG_ENV 1
 #endif
 #if defined RUBY_DEBUG_ENV && !RUBY_DEBUG_ENV
 # undef RUBY_DEBUG_ENV
 #endif
 
+static int
+rb_main(int argc, char **argv)
+{
+    RUBY_INIT_STACK;
+    ruby_init();
+    return ruby_run_node(ruby_options(argc, argv));
+}
+
+#if defined(__wasm__) && !defined(__EMSCRIPTEN__)
+int rb_wasm_rt_start(int (main)(int argc, char **argv), int argc, char **argv);
+#define rb_main(argc, argv) rb_wasm_rt_start(rb_main, argc, argv)
+#endif
+
 int
 main(int argc, char **argv)
 {
@@ -41,9 +54,5 @@ main(int argc, char **argv)
 #endif
 
     ruby_sysinit(&argc, &argv);
-    {
-	RUBY_INIT_STACK;
-	ruby_init();
-	return ruby_run_node(ruby_options(argc, argv));
-    }
+    return rb_main(argc, argv);
 }
