@@ -139,14 +139,19 @@ RSpec.describe Bundler::Dsl do
   describe "#gem" do
     # rubocop:disable Naming/VariableNumber
     [:ruby, :ruby_18, :ruby_19, :ruby_20, :ruby_21, :ruby_22, :ruby_23, :ruby_24, :ruby_25, :ruby_26, :ruby_27,
-     :ruby_30, :ruby_31, :mri, :mri_18, :mri_19, :mri_20, :mri_21, :mri_22, :mri_23, :mri_24, :mri_25, :mri_26,
-     :mri_27, :mri_30, :mri_31, :jruby, :rbx, :truffleruby].each do |platform|
+     :ruby_30, :ruby_31, :ruby_32, :ruby_33, :mri, :mri_18, :mri_19, :mri_20, :mri_21, :mri_22, :mri_23, :mri_24,
+     :mri_25, :mri_26, :mri_27, :mri_30, :mri_31, :mri_32, :mri_33, :jruby, :rbx, :truffleruby].each do |platform|
       it "allows #{platform} as a valid platform" do
         subject.gem("foo", :platform => platform)
       end
     end
     # rubocop:enable Naming/VariableNumber
 
+    it "allows platforms matching the running Ruby version" do
+      platform = "ruby_#{RbConfig::CONFIG["MAJOR"]}#{RbConfig::CONFIG["MINOR"]}"
+      subject.gem("foo", :platform => platform)
+    end
+
     it "rejects invalid platforms" do
       expect { subject.gem("foo", :platform => :bogus) }.
         to raise_error(Bundler::GemfileError, /is not a valid platform/)
