@@ -13,18 +13,18 @@ RSpec.describe "gemcutter's dependency API", :realworld => true do
       require_relative "../support/artifice/endpoint_timeout"
 
       @t = Thread.new do
-        server = Rack::Server.start(:app       => EndpointTimeout,
-                                    :Host      => "0.0.0.0",
-                                    :Port      => port,
-                                    :server    => "webrick",
+        server = Rack::Server.start(:app => EndpointTimeout,
+                                    :Host => "0.0.0.0",
+                                    :Port => port,
+                                    :server => "webrick",
                                     :AccessLog => [],
-                                    :Logger    => Spec::SilentLogger.new)
+                                    :Logger => Spec::SilentLogger.new)
         server.start
       end
       @t.run
 
       wait_for_server("127.0.0.1", port)
-      bundle! "config set timeout 1"
+      bundle "config set timeout 1"
     end
 
     after do
@@ -34,7 +34,7 @@ RSpec.describe "gemcutter's dependency API", :realworld => true do
     end
 
     it "times out and falls back on the modern index" do
-      install_gemfile! <<-G, :artifice => nil
+      install_gemfile <<-G, :artifice => nil
         source "#{@server_uri}"
         gem "rack"
       G
