@@ -1,8 +1,6 @@
 # frozen_string_literal: true
 
-require_relative "compact_index"
-
-Artifice.deactivate
+require_relative "helpers/compact_index"
 
 class CompactIndexPartialUpdate < CompactIndexAPI
   # Stub the server to never return 304s. This simulates the behaviour of
@@ -18,21 +16,23 @@ class CompactIndexPartialUpdate < CompactIndexAPI
     )
 
     # Verify a cached copy of the versions file exists
-    unless File.read(cached_versions_path).start_with?("created_at: ")
+    unless File.binread(cached_versions_path).start_with?("created_at: ")
       raise("Cached versions file should be present and have content")
     end
 
     # Verify that a partial request is made, starting from the index of the
     # final byte of the cached file.
-    unless env["HTTP_RANGE"] == "bytes=#{File.read(cached_versions_path).bytesize - 1}-"
+    unless env["HTTP_RANGE"] == "bytes=#{File.binread(cached_versions_path).bytesize - 1}-"
       raise("Range header should be present, and start from the index of the final byte of the cache.")
     end
 
     etag_response do
       # Return the exact contents of the cache.
-      File.read(cached_versions_path)
+      File.binread(cached_versions_path)
     end
   end
 end
 
+require_relative "helpers/artifice"
+
 Artifice.activate_with(CompactIndexPartialUpdate)
