@@ -1,6 +1,6 @@
 # frozen_string_literal: true
 
-require_relative "endpoint"
+require_relative "helpers/endpoint"
 
 class EndpointMirrorSource < Endpoint
   get "/gems/:id" do
@@ -12,4 +12,6 @@ class EndpointMirrorSource < Endpoint
   end
 end
 
+require_relative "helpers/artifice"
+
 Artifice.activate_with(EndpointMirrorSource)
