@@ -27,13 +27,13 @@ describe "Hash#to_a" do
     ent.should == pairs
   end
 
-  ruby_version_is ''...'2.7' do
-    it "returns a tainted array if self is tainted" do
-      {}.taint.to_a.tainted?.should be_true
+  ruby_version_is ''...'3.0' do
+    it "returns a not tainted array if self is tainted" do
+      {}.taint.to_a.tainted?.should be_false
     end
 
-    it "returns an untrusted array if self is untrusted" do
-      {}.untrust.to_a.untrusted?.should be_true
+    it "returns a trusted array if self is untrusted" do
+      {}.untrust.to_a.untrusted?.should be_false
     end
   end
 end
