@@ -36,35 +36,23 @@ describe "Kernel.proc" do
 end
 
 describe "Kernel#proc" do
-  ruby_version_is ""..."2.7" do
-    it "uses the implicit block from an enclosing method" do
-      def some_method
-        proc
-      end
-
-      prc = some_method { "hello" }
-
-      prc.call.should == "hello"
-    end
+  def some_method
+    proc
   end
 
-  ruby_version_is "2.7" do
-    before :each do
-      @deprecated = Warning[:deprecated]
-      Warning[:deprecated] = true
-    end
-    after :each do
-      Warning[:deprecated] = @deprecated
-    end
-
+  ruby_version_is ""..."3.0" do
     it "can be created when called with no block" do
-      def some_method
-        proc
-      end
-
       -> {
         some_method { "hello" }
       }.should complain(/Capturing the given block using Kernel#proc is deprecated/)
     end
   end
+
+  ruby_version_is "3.0" do
+    it "raises an ArgumentError when passed no block" do
+      -> {
+        some_method { "hello" }
+      }.should raise_error(ArgumentError, 'tried to create Proc object without a block')
+    end
+  end
 end
