@@ -2,6 +2,6 @@ require_relative '../../spec_helper'
 
 describe "NilClass#nil?" do
   it "returns true" do
-    nil.nil?.should == true
+    nil.should.nil?
   end
 end
