@@ -2,7 +2,7 @@ require_relative '../../spec_helper'
 
 describe "Regexp#initialize" do
   it "is a private method" do
-    Regexp.should have_private_method(:initialize)
+    Regexp.should have_private_instance_method(:initialize)
   end
 
   ruby_version_is ""..."3.0" do
