@@ -40,4 +40,12 @@ describe "Struct#initialize" do
   it "can be overridden" do
     StructClasses::SubclassX.new(:y).new.key.should == :value
   end
+
+  ruby_version_is "3.1"..."3.2" do
+    it "warns about passing only keyword arguments" do
+      -> {
+        StructClasses::Ruby.new(version: "3.1", platform: "OS")
+      }.should complain(/warning: Passing only keyword arguments/)
+    end
+  end
 end
