@@ -571,6 +571,13 @@ describe "A method" do
       end
     end
 
+    evaluate <<-ruby do
+        def m(a:, **kw) [a, kw] end
+      ruby
+
+      -> { m(b: 1) }.should raise_error(ArgumentError)
+    end
+
     evaluate <<-ruby do
         def m(a: 1) a end
       ruby
@@ -602,13 +609,11 @@ describe "A method" do
       -> { m(2) }.should raise_error(ArgumentError)
     end
 
-    ruby_version_is "2.7" do
-      evaluate <<-ruby do
-        def m(**k); k end;
-        ruby
+    evaluate <<-ruby do
+      def m(**k); k end;
+      ruby
 
-        m("a" => 1).should == { "a" => 1 }
-      end
+      m("a" => 1).should == { "a" => 1 }
     end
 
     evaluate <<-ruby do
@@ -962,26 +967,13 @@ describe "A method" do
       end
     end
 
-    ruby_version_is ""..."2.7" do
-      evaluate <<-ruby do
-          def m(a=1, **) a end
-        ruby
+    evaluate <<-ruby do
+        def m(a=1, **) a end
+      ruby
 
-        m().should == 1
-        m(2, a: 1, b: 0).should == 2
-        m("a" => 1, a: 2).should == {"a" => 1}
-      end
-    end
-
-    ruby_version_is "2.7" do
-      evaluate <<-ruby do
-          def m(a=1, **) a end
-        ruby
-
-        m().should == 1
-        m(2, a: 1, b: 0).should == 2
-        m("a" => 1, a: 2).should == 1
-      end
+      m().should == 1
+      m(2, a: 1, b: 0).should == 2
+      m("a" => 1, a: 2).should == 1
     end
 
     evaluate <<-ruby do
@@ -1021,164 +1013,7 @@ describe "A method" do
       m(1, 2, 3).should == [[1, 2], 3]
     end
 
-    ruby_version_is ""..."2.7" do
-      evaluate <<-ruby do
-          def m(*, a:) a end
-        ruby
-
-        m(a: 1).should == 1
-        m(1, 2, a: 3).should == 3
-        suppress_keyword_warning do
-          m("a" => 1, a: 2).should == 2
-        end
-      end
-
-      evaluate <<-ruby do
-          def m(*a, b:) [a, b] end
-        ruby
-
-        m(b: 1).should == [[], 1]
-        m(1, 2, b: 3).should == [[1, 2], 3]
-        suppress_keyword_warning do
-          m("a" => 1, b: 2).should == [[{"a" => 1}], 2]
-        end
-      end
-
-      evaluate <<-ruby do
-          def m(*, a: 1) a end
-        ruby
-
-        m().should == 1
-        m(1, 2).should == 1
-        m(a: 2).should == 2
-        m(1, a: 2).should == 2
-        suppress_keyword_warning do
-          m("a" => 1, a: 2).should == 2
-        end
-      end
-
-      evaluate <<-ruby do
-          def m(*a, b: 1) [a, b] end
-        ruby
-
-        m().should == [[], 1]
-        m(1, 2, 3, b: 4).should == [[1, 2, 3], 4]
-        suppress_keyword_warning do
-          m("a" => 1, b: 2).should == [[{"a" => 1}], 2]
-        end
-
-        a = mock("splat")
-        a.should_not_receive(:to_ary)
-        m(*a).should == [[a], 1]
-      end
-
-      evaluate <<-ruby do
-          def m(*, **) end
-        ruby
-
-        m().should be_nil
-        m(a: 1, b: 2).should be_nil
-        m(1, 2, 3, a: 4, b: 5).should be_nil
-
-        h = mock("keyword splat")
-        h.should_receive(:to_hash).and_return({a: 1})
-        suppress_keyword_warning do
-          m(h).should be_nil
-        end
-
-        h = mock("keyword splat")
-        error = RuntimeError.new("error while converting to a hash")
-        h.should_receive(:to_hash).and_raise(error)
-        -> { m(h) }.should raise_error(error)
-      end
-
-      evaluate <<-ruby do
-          def m(*a, **) a end
-        ruby
-
-        m().should == []
-        m(1, 2, 3, a: 4, b: 5).should == [1, 2, 3]
-        m("a" => 1, a: 1).should == [{"a" => 1}]
-        m(1, **{a: 2}).should == [1]
-
-        h = mock("keyword splat")
-        h.should_receive(:to_hash)
-        -> { m(**h) }.should raise_error(TypeError)
-      end
-
-      evaluate <<-ruby do
-          def m(*, **k) k end
-        ruby
-
-        m().should == {}
-        m(1, 2, 3, a: 4, b: 5).should == {a: 4, b: 5}
-        m("a" => 1, a: 1).should == {a: 1}
-
-        h = mock("keyword splat")
-        h.should_receive(:to_hash).and_return({a: 1})
-        m(h).should == {a: 1}
-      end
-
-      evaluate <<-ruby do
-          def m(a = nil, **k) [a, k] end
-        ruby
-
-        m().should == [nil, {}]
-        m("a" => 1).should == [{"a" => 1}, {}]
-        m(a: 1).should == [nil, {a: 1}]
-        m("a" => 1, a: 1).should == [{"a" => 1}, {a: 1}]
-        m({ "a" => 1 }, a: 1).should == [{"a" => 1}, {a: 1}]
-        m({a: 1}, {}).should == [{a: 1}, {}]
-
-        h = {"a" => 1, b: 2}
-        m(h).should == [{"a" => 1}, {b: 2}]
-        h.should == {"a" => 1, b: 2}
-
-        h = {"a" => 1}
-        m(h).first.should == h
-
-        h = {}
-        r = m(h)
-        r.first.should be_nil
-        r.last.should == {}
-
-        hh = {}
-        h = mock("keyword splat empty hash")
-        h.should_receive(:to_hash).and_return(hh)
-        r = m(h)
-        r.first.should be_nil
-        r.last.should == {}
-
-        h = mock("keyword splat")
-        h.should_receive(:to_hash).and_return({"a" => 1, a: 2})
-        m(h).should == [{"a" => 1}, {a: 2}]
-      end
-
-      evaluate <<-ruby do
-          def m(*a, **k) [a, k] end
-        ruby
-
-        m().should == [[], {}]
-        m(1).should == [[1], {}]
-        m(a: 1, b: 2).should == [[], {a: 1, b: 2}]
-        m(1, 2, 3, a: 2).should == [[1, 2, 3], {a: 2}]
-
-        m("a" => 1).should == [[{"a" => 1}], {}]
-        m(a: 1).should == [[], {a: 1}]
-        m("a" => 1, a: 1).should == [[{"a" => 1}], {a: 1}]
-        m({ "a" => 1 }, a: 1).should == [[{"a" => 1}], {a: 1}]
-        m({a: 1}, {}).should == [[{a: 1}], {}]
-        m({a: 1}, {"a" => 1}).should == [[{a: 1}, {"a" => 1}], {}]
-
-        bo = BasicObject.new
-        def bo.to_a; [1, 2, 3]; end
-        def bo.to_hash; {:b => 2, :c => 3}; end
-
-        m(*bo, **bo).should == [[1, 2, 3], {:b => 2, :c => 3}]
-      end
-    end
-
-    ruby_version_is "2.7"...'3.0' do
+    ruby_version_is ""...'3.0' do
       evaluate <<-ruby do
           def m(*, a:) a end
         ruby
@@ -1503,44 +1338,22 @@ describe "A method" do
       end
     end
 
-    ruby_version_is ''...'2.7' do
-      evaluate <<-ruby do
-          def m(a:, **) a end
-        ruby
+    evaluate <<-ruby do
+        def m(a:, **) a end
+      ruby
 
-        m(a: 1).should == 1
-        m(a: 1, b: 2).should == 1
-        -> { m("a" => 1, a: 1, b: 2) }.should raise_error(ArgumentError)
-      end
-
-      evaluate <<-ruby do
-          def m(a:, **k) [a, k] end
-        ruby
-
-        m(a: 1).should == [1, {}]
-        m(a: 1, b: 2, c: 3).should == [1, {b: 2, c: 3}]
-        -> { m("a" => 1, a: 1, b: 2) }.should raise_error(ArgumentError)
-      end
+      m(a: 1).should == 1
+      m(a: 1, b: 2).should == 1
+      m("a" => 1, a: 1, b: 2).should == 1
     end
 
-    ruby_version_is '2.7' do
-      evaluate <<-ruby do
-          def m(a:, **) a end
-        ruby
+    evaluate <<-ruby do
+        def m(a:, **k) [a, k] end
+      ruby
 
-        m(a: 1).should == 1
-        m(a: 1, b: 2).should == 1
-        m("a" => 1, a: 1, b: 2).should == 1
-      end
-
-      evaluate <<-ruby do
-          def m(a:, **k) [a, k] end
-        ruby
-
-        m(a: 1).should == [1, {}]
-        m(a: 1, b: 2, c: 3).should == [1, {b: 2, c: 3}]
-        m("a" => 1, a: 1, b: 2).should == [1, {"a" => 1, b: 2}]
-      end
+      m(a: 1).should == [1, {}]
+      m(a: 1, b: 2, c: 3).should == [1, {b: 2, c: 3}]
+      m("a" => 1, a: 1, b: 2).should == [1, {"a" => 1, b: 2}]
     end
 
     evaluate <<-ruby do
@@ -1637,18 +1450,16 @@ describe "A method" do
       result.should == [1, 1, [], 2, 3, 2, 4, { h: 5, i: 6 }, l]
     end
 
-    ruby_version_is "2.7" do
-      evaluate <<-ruby do
-        def m(a, **nil); a end;
-        ruby
+    evaluate <<-ruby do
+      def m(a, **nil); a end;
+      ruby
 
-        m({a: 1}).should == {a: 1}
-        m({"a" => 1}).should == {"a" => 1}
+      m({a: 1}).should == {a: 1}
+      m({"a" => 1}).should == {"a" => 1}
 
-        -> { m(a: 1) }.should raise_error(ArgumentError)
-        -> { m(**{a: 1}) }.should raise_error(ArgumentError)
-        -> { m("a" => 1) }.should raise_error(ArgumentError)
-      end
+      -> { m(a: 1) }.should raise_error(ArgumentError)
+      -> { m(**{a: 1}) }.should raise_error(ArgumentError)
+      -> { m("a" => 1) }.should raise_error(ArgumentError)
     end
 
     ruby_version_is ''...'3.0' do
@@ -1690,19 +1501,17 @@ describe "A method" do
     end
   end
 
-  ruby_version_is '2.7' do
-    context 'when passing an empty keyword splat to a method that does not accept keywords' do
-      evaluate <<-ruby do
-          def m(*a); a; end
-        ruby
+  context 'when passing an empty keyword splat to a method that does not accept keywords' do
+    evaluate <<-ruby do
+        def m(*a); a; end
+      ruby
 
-        h = {}
-        m(**h).should == []
-      end
+      h = {}
+      m(**h).should == []
     end
   end
 
-  ruby_version_is '2.7'...'3.0' do
+  ruby_version_is ''...'3.0' do
     context 'when passing an empty keyword splat to a method that does not accept keywords' do
       evaluate <<-ruby do
           def m(a); a; end
@@ -1870,6 +1679,15 @@ ruby_version_is "3.0" do
 
         m.should == 42
       end
+
+      context "without parenthesis" do
+        evaluate <<-ruby do
+          def m = 42
+        ruby
+
+          m.should == 42
+        end
+      end
     end
 
     context "with arguments" do
@@ -1907,6 +1725,16 @@ ruby_version_is "3.0" do
         m("meow", num: 2).should == "meow" * 4
       end
     end
+
+    ruby_version_is ""..."3.0" do
+      context "inside 'endless' method definitions" do
+        it "does not allow method calls without parenthesis" do
+          -> {
+            eval("def greet(person) = 'Hi, '.concat person")
+          }.should raise_error(SyntaxError)
+        end
+      end
+    end
   end
 
   describe "Keyword arguments are now separated from positional arguments" do
@@ -2015,4 +1843,14 @@ ruby_version_is "3.1" do
       end
     end
   end
+
+  describe "Inside 'endless' method definitions" do
+    it "allows method calls without parenthesis" do
+      eval <<-ruby
+        def greet(person) = "Hi, ".concat person
+      ruby
+
+      greet("Homer").should == "Hi, Homer"
+    end
+  end
 end
