@@ -258,20 +258,10 @@ describe "Invoking a private setter method" do
 end
 
 describe "Invoking a private getter method" do
-  ruby_version_is ""..."2.7" do
-    it "does not permit self as a receiver" do
-      receiver = LangSendSpecs::PrivateGetter.new
-      -> { receiver.call_self_foo }.should raise_error(NoMethodError)
-      -> { receiver.call_self_foo_or_equals(6) }.should raise_error(NoMethodError)
-    end
-  end
-
-  ruby_version_is "2.7" do
-    it "permits self as a receiver" do
-      receiver = LangSendSpecs::PrivateGetter.new
-      receiver.call_self_foo_or_equals(6)
-      receiver.call_self_foo.should == 6
-    end
+  it "permits self as a receiver" do
+    receiver = LangSendSpecs::PrivateGetter.new
+    receiver.call_self_foo_or_equals(6)
+    receiver.call_self_foo.should == 6
   end
 end
 
@@ -421,18 +411,36 @@ describe "Invoking a method" do
     specs.rest_len(0,*a,4,*5,6,7,*c,-1).should == 11
   end
 
-  it "expands the Array elements from the splat after executing the arguments and block if no other arguments follow the splat" do
-    def self.m(*args, &block)
-      [args, block]
+  ruby_version_is ""..."3.0" do
+    it "expands the Array elements from the splat after executing the arguments and block if no other arguments follow the splat" do
+      def self.m(*args, &block)
+        [args, block]
+      end
+
+      args = [1, nil]
+      m(*args, &args.pop).should == [[1], nil]
+
+      args = [1, nil]
+      order = []
+      m(*(order << :args; args), &(order << :block; args.pop)).should == [[1], nil]
+      order.should == [:args, :block]
     end
+  end
 
-    args = [1, nil]
-    m(*args, &args.pop).should == [[1], nil]
+  ruby_version_is "3.0" do
+    it "expands the Array elements from the splat before applying block argument operations" do
+      def self.m(*args, &block)
+        [args, block]
+      end
 
-    args = [1, nil]
-    order = []
-    m(*(order << :args; args), &(order << :block; args.pop)).should == [[1], nil]
-    order.should == [:args, :block]
+      args = [1, nil]
+      m(*args, &args.pop).should == [[1, nil], nil]
+
+      args = [1, nil]
+      order = []
+      m(*(order << :args; args), &(order << :block; args.pop)).should == [[1, nil], nil]
+      order.should == [:args, :block]
+    end
   end
 
   it "evaluates the splatted arguments before the block if there are other arguments after the splat" do
