@@ -1,4 +1,4 @@
-require File.expand_path('../spec_helper', __FILE__)
+require_relative 'spec_helper'
 
 with_feature :readline do
   describe "Readline.completer_word_break_characters" do
