@@ -108,17 +108,37 @@ describe "C-API Class function" do
 
   describe "rb_class_new_instance" do
     it "allocates and initializes a new object" do
-      o = @s.rb_class_new_instance(0, nil, CApiClassSpecs::Alloc)
+      o = @s.rb_class_new_instance([], CApiClassSpecs::Alloc)
       o.class.should == CApiClassSpecs::Alloc
       o.initialized.should be_true
     end
 
     it "passes arguments to the #initialize method" do
-      o = @s.rb_class_new_instance(2, [:one, :two], CApiClassSpecs::Alloc)
+      o = @s.rb_class_new_instance([:one, :two], CApiClassSpecs::Alloc)
       o.arguments.should == [:one, :two]
     end
   end
 
+  ruby_version_is "3.0" do
+    describe "rb_class_new_instance_kw" do
+      it "passes arguments and keywords to the #initialize method" do
+        obj = @s.rb_class_new_instance_kw([{pos: 1}, {kw: 2}], CApiClassSpecs::KeywordAlloc)
+        obj.args.should == [{pos: 1}]
+        obj.kwargs.should == {kw: 2}
+
+        obj = @s.rb_class_new_instance_kw([{}], CApiClassSpecs::KeywordAlloc)
+        obj.args.should == []
+        obj.kwargs.should == {}
+      end
+
+      it "raises TypeError if the last argument is not a Hash" do
+        -> {
+          @s.rb_class_new_instance_kw([42], CApiClassSpecs::KeywordAlloc)
+        }.should raise_error(TypeError, 'no implicit conversion of Integer into Hash')
+      end
+    end
+  end
+
   describe "rb_include_module" do
     it "includes a module into a class" do
       c = Class.new
@@ -176,16 +196,6 @@ describe "C-API Class function" do
     end
   end
 
-  describe "rb_define_method" do
-    it "defines a method taking variable arguments as a C array if the argument count is -1" do
-      @s.rb_method_varargs_1(1, 3, 7, 4).should == [1, 3, 7, 4]
-    end
-
-    it "defines a method taking variable arguments as a Ruby array if the argument count is -2" do
-      @s.rb_method_varargs_2(1, 3, 7, 4).should == [1, 3, 7, 4]
-    end
-  end
-
   describe "rb_class2name" do
     it "returns the class name" do
       @s.rb_class2name(CApiClassSpecs).should == "CApiClassSpecs"
@@ -194,6 +204,10 @@ describe "C-API Class function" do
     it "returns a string for an anonymous class" do
       @s.rb_class2name(Class.new).should be_kind_of(String)
     end
+
+    it "returns a string beginning with # for an anonymous class" do
+      @s.rb_class2name(Struct.new(:x, :y).new(1, 2).class).should.start_with?('#')
+    end
   end
 
   describe "rb_class_path" do
@@ -309,6 +323,15 @@ describe "C-API Class function" do
         @s.rb_define_class("ClassSpecDefineClass4", nil)
       }.should raise_error(ArgumentError)
     end
+
+    it "allows arbitrary names, including constant names not valid in Ruby" do
+      cls = @s.rb_define_class("_INVALID_CLASS", CApiClassSpecs::Super)
+      cls.name.should == "_INVALID_CLASS"
+
+      -> {
+        Object.const_get(cls.name)
+      }.should raise_error(NameError, /wrong constant name/)
+    end
   end
 
   describe "rb_define_class_under" do
@@ -353,6 +376,15 @@ describe "C-API Class function" do
     it "raises a TypeError if class is defined and its superclass mismatches the given one" do
       -> { @s.rb_define_class_under(CApiClassSpecs, "Sub", Object) }.should raise_error(TypeError)
     end
+
+    it "allows arbitrary names, including constant names not valid in Ruby" do
+      cls = @s.rb_define_class_under(CApiClassSpecs, "_INVALID_CLASS", CApiClassSpecs::Super)
+      cls.name.should == "CApiClassSpecs::_INVALID_CLASS"
+
+      -> {
+        CApiClassSpecs.const_get(cls.name)
+      }.should raise_error(NameError, /wrong constant name/)
+    end
   end
 
   describe "rb_define_class_id_under" do
@@ -380,6 +412,15 @@ describe "C-API Class function" do
     it "raises a TypeError if class is defined and its superclass mismatches the given one" do
       -> { @s.rb_define_class_id_under(CApiClassSpecs, :Sub, Object) }.should raise_error(TypeError)
     end
+
+    it "allows arbitrary names, including constant names not valid in Ruby" do
+      cls = @s.rb_define_class_id_under(CApiClassSpecs, :_INVALID_CLASS2, CApiClassSpecs::Super)
+      cls.name.should == "CApiClassSpecs::_INVALID_CLASS2"
+
+      -> {
+        CApiClassSpecs.const_get(cls.name)
+      }.should raise_error(NameError, /wrong constant name/)
+    end
   end
 
   describe "rb_define_class_variable" do
