@@ -118,11 +118,14 @@ static VALUE so_rb_obj_call_init(VALUE self, VALUE object,
   return Qnil;
 }
 
+static VALUE so_rb_obj_class(VALUE self, VALUE obj) {
+  return rb_obj_class(obj);
+}
+
 static VALUE so_rbobjclassname(VALUE self, VALUE obj) {
   return rb_str_new2(rb_obj_classname(obj));
 }
 
-
 static VALUE object_spec_rb_obj_freeze(VALUE self, VALUE obj) {
   return rb_obj_freeze(obj);
 }
@@ -161,9 +164,11 @@ static VALUE object_specs_rb_obj_method(VALUE self, VALUE obj, VALUE method) {
 # endif
 #endif
 
+#ifndef RUBY_VERSION_IS_3_2
 static VALUE object_spec_rb_obj_taint(VALUE self, VALUE obj) {
   return rb_obj_taint(obj);
 }
+#endif
 
 #if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
 # pragma GCC diagnostic pop
@@ -296,6 +301,13 @@ static VALUE so_is_rb_type_p_data(VALUE self, VALUE obj) {
   return Qfalse;
 }
 
+static VALUE so_is_rb_type_p_file(VALUE self, VALUE obj) {
+  if(rb_type_p(obj, T_FILE)) {
+    return Qtrue;
+  }
+  return Qfalse;
+}
+
 static VALUE so_is_builtin_type_object(VALUE self, VALUE obj) {
   if(BUILTIN_TYPE(obj) == T_OBJECT) {
     return Qtrue;
@@ -381,6 +393,18 @@ static VALUE object_spec_rb_class_inherited_p(VALUE self, VALUE mod, VALUE arg)
   return rb_class_inherited_p(mod, arg);
 }
 
+static int foreach_f(ID key, VALUE val, VALUE ary) {
+  rb_ary_push(ary, ID2SYM(key));
+  rb_ary_push(ary, val);
+  return ST_CONTINUE;
+}
+
+static VALUE object_spec_rb_ivar_foreach(VALUE self, VALUE obj) {
+  VALUE ary = rb_ary_new();
+  rb_ivar_foreach(obj, foreach_f, ary);
+  return ary;
+}
+
 static VALUE speced_allocator(VALUE klass) {
   VALUE flags = 0;
   VALUE instance;
@@ -440,6 +464,7 @@ void Init_object_spec(void) {
   rb_define_method(cls, "rb_obj_alloc", so_rb_obj_alloc, 1);
   rb_define_method(cls, "rb_obj_dup", so_rb_obj_dup, 1);
   rb_define_method(cls, "rb_obj_call_init", so_rb_obj_call_init, 3);
+  rb_define_method(cls, "rb_obj_class", so_rb_obj_class, 1);
   rb_define_method(cls, "rb_obj_classname", so_rbobjclassname, 1);
   rb_define_method(cls, "rb_obj_freeze", object_spec_rb_obj_freeze, 1);
   rb_define_method(cls, "rb_obj_frozen_p", object_spec_rb_obj_frozen_p, 1);
@@ -448,7 +473,9 @@ void Init_object_spec(void) {
   rb_define_method(cls, "rb_obj_is_kind_of", so_kind_of, 2);
   rb_define_method(cls, "rb_obj_method_arity", object_specs_rb_obj_method_arity, 2);
   rb_define_method(cls, "rb_obj_method", object_specs_rb_obj_method, 2);
+#ifndef RUBY_VERSION_IS_3_2
   rb_define_method(cls, "rb_obj_taint", object_spec_rb_obj_taint, 1);
+#endif
   rb_define_method(cls, "rb_require", so_require, 0);
   rb_define_method(cls, "rb_respond_to", so_respond_to, 2);
   rb_define_method(cls, "rb_method_boundp", object_spec_rb_method_boundp, 3);
@@ -470,6 +497,7 @@ void Init_object_spec(void) {
   rb_define_method(cls, "rb_is_rb_type_p_module", so_is_rb_type_p_module, 1);
   rb_define_method(cls, "rb_is_rb_type_p_class", so_is_rb_type_p_class, 1);
   rb_define_method(cls, "rb_is_rb_type_p_data", so_is_rb_type_p_data, 1);
+  rb_define_method(cls, "rb_is_rb_type_p_file", so_is_rb_type_p_file, 1);
   rb_define_method(cls, "rb_is_builtin_type_object", so_is_builtin_type_object, 1);
   rb_define_method(cls, "rb_is_builtin_type_array", so_is_builtin_type_array, 1);
   rb_define_method(cls, "rb_is_builtin_type_module", so_is_builtin_type_module, 1);
@@ -492,6 +520,7 @@ void Init_object_spec(void) {
   rb_define_method(cls, "speced_allocator?", speced_allocator_p, 1);
   rb_define_method(cls, "custom_alloc_func?", custom_alloc_func_p, 1);
   rb_define_method(cls, "not_implemented_method", rb_f_notimplement, -1);
+  rb_define_method(cls, "rb_ivar_foreach", object_spec_rb_ivar_foreach, 1);
 }
 
 #ifdef __cplusplus
