@@ -1,5 +1,5 @@
-# frozen_string_literal: false
-require 'rdoc/test_case'
+# frozen_string_literal: true
+require_relative 'helper'
 
 class TestRDocMarkupAttributeManager < RDoc::TestCase
 
@@ -36,12 +36,12 @@ class TestRDocMarkupAttributeManager < RDoc::TestCase
   end
 
   def crossref(text)
-    crossref_bitmap = @am.attributes.bitmap_for(:_SPECIAL_) |
+    crossref_bitmap = @am.attributes.bitmap_for(:_REGEXP_HANDLING_) |
                       @am.attributes.bitmap_for(:CROSSREF)
 
-    [ @am.changed_attribute_by_name([], [:CROSSREF, :_SPECIAL_]),
-      RDoc::Markup::Special.new(crossref_bitmap, text),
-      @am.changed_attribute_by_name([:CROSSREF, :_SPECIAL_], [])
+    [ @am.changed_attribute_by_name([], [:CROSSREF, :_REGEXP_HANDLING_]),
+      RDoc::Markup::RegexpHandling.new(crossref_bitmap, text),
+      @am.changed_attribute_by_name([:CROSSREF, :_REGEXP_HANDLING_], [])
     ]
   end
 
@@ -58,12 +58,12 @@ class TestRDocMarkupAttributeManager < RDoc::TestCase
     assert(tags.has_key?("test"))
   end
 
-  def test_add_special
-    @am.add_special "WikiWord", :WIKIWORD
-    specials = @am.special
+  def test_add_regexp_handling
+    @am.add_regexp_handling "WikiWord", :WIKIWORD
+    regexp_handlings = @am.regexp_handlings
 
-    assert_equal 1, specials.size
-    assert specials.assoc "WikiWord"
+    assert_equal 1, regexp_handlings.size
+    assert regexp_handlings.assoc "WikiWord"
   end
 
   def test_add_word_pair
@@ -75,7 +75,7 @@ class TestRDocMarkupAttributeManager < RDoc::TestCase
   end
 
   def test_add_word_pair_angle
-    e = assert_raises ArgumentError do
+    e = assert_raise ArgumentError do
       @am.add_word_pair '<', '>', 'angles'
     end
 
@@ -83,7 +83,7 @@ class TestRDocMarkupAttributeManager < RDoc::TestCase
   end
 
   def test_add_word_pair_invalid
-    assert_raises ArgumentError do
+    assert_raise ArgumentError do
       @am.add_word_pair("<", "<", :TEST)
     end
   end
@@ -145,6 +145,8 @@ class TestRDocMarkupAttributeManager < RDoc::TestCase
 
     assert_equal(["cat and ", @em_on, "5", @em_off, " dogs"],
                   @am.flow("cat and _5_ dogs"))
+
+    assert_equal([@tt_on, "__id__", @tt_off], @am.flow("+__id__+"))
   end
 
   def test_bold
@@ -171,23 +173,26 @@ class TestRDocMarkupAttributeManager < RDoc::TestCase
   end
 
   def test_convert_attrs
-    str = '+foo+'
-    attrs = RDoc::Markup::AttrSpan.new str.length
+    str = '+foo+'.dup
+    attrs = RDoc::Markup::AttrSpan.new str.length, @am.exclusive_bitmap
 
+    @am.convert_attrs str, attrs, true
     @am.convert_attrs str, attrs
 
     assert_equal "\000foo\000", str
 
-    str = '+:foo:+'
-    attrs = RDoc::Markup::AttrSpan.new str.length
+    str = '+:foo:+'.dup
+    attrs = RDoc::Markup::AttrSpan.new str.length, @am.exclusive_bitmap
 
+    @am.convert_attrs str, attrs, true
     @am.convert_attrs str, attrs
 
     assert_equal "\000:foo:\000", str
 
-    str = '+x-y+'
-    attrs = RDoc::Markup::AttrSpan.new str.length
+    str = '+x-y+'.dup
+    attrs = RDoc::Markup::AttrSpan.new str.length, @am.exclusive_bitmap
 
+    @am.convert_attrs str, attrs, true
     @am.convert_attrs str, attrs
 
     assert_equal "\000x-y\000", str
@@ -243,6 +248,22 @@ class TestRDocMarkupAttributeManager < RDoc::TestCase
                  output('unhandled <p>tag</p> unchanged')
   end
 
+  def test_exclude_tag
+    assert_equal '<CODE>aaa</CODE>[:symbol]', output('+aaa+[:symbol]')
+    assert_equal '<CODE>aaa[:symbol]</CODE>', output('+aaa[:symbol]+')
+    assert_equal 'aaa[:symbol]', output('aaa[:symbol]')
+    assert_equal '<B><CODE>index</CODE></B>', output('<b><tt>index</tt></b>')
+  end
+
+  def test_exclude_tag_flow
+    assert_equal [@tt_on, "aaa", @tt_off, "[:symbol]"],
+                  @am.flow("+aaa+[:symbol]")
+    assert_equal [@tt_on, "aaa[:symbol]", @tt_off],
+                  @am.flow("+aaa[:symbol]+")
+    assert_equal ["aaa[:symbol]"],
+                  @am.flow("aaa[:symbol]")
+  end
+
   def test_html_like_em_bold
     assert_equal ["cat ", @em_on, "and ", @em_to_bold, "dog", @bold_off],
                   @am.flow("cat <i>and </i><b>dog</b>")
@@ -299,17 +320,17 @@ class TestRDocMarkupAttributeManager < RDoc::TestCase
     def @am.str()     @str       end
     def @am.str=(str) @str = str end
 
-    @am.str = '<code>foo</code>'
+    @am.str = '<code>foo</code>'.dup
     @am.mask_protected_sequences
 
     assert_equal "<code>foo</code>",       @am.str
 
-    @am.str = '<code>foo\\</code>'
+    @am.str = '<code>foo\\</code>'.dup
     @am.mask_protected_sequences
 
     assert_equal "<code>foo<\x04/code>", @am.str, 'escaped close'
 
-    @am.str = '<code>foo\\\\</code>'
+    @am.str = '<code>foo\\\\</code>'.dup
     @am.mask_protected_sequences
 
     assert_equal "<code>foo\\</code>",     @am.str, 'escaped backslash'
@@ -332,8 +353,16 @@ class TestRDocMarkupAttributeManager < RDoc::TestCase
                   @am.flow("\\_cat_<i>dog</i>"))
   end
 
-  def test_special
-    @am.add_special(RDoc::CrossReference::CROSSREF_REGEXP, :CROSSREF)
+  def test_lost_tag_for_the_second_time
+    str = "cat <tt>dog</tt>"
+    assert_equal(["cat ", @tt_on, "dog", @tt_off],
+                 @am.flow(str))
+    assert_equal(["cat ", @tt_on, "dog", @tt_off],
+                 @am.flow(str))
+  end
+
+  def test_regexp_handling
+    @am.add_regexp_handling(RDoc::CrossReference::CROSSREF_REGEXP, :CROSSREF)
 
     #
     # The apostrophes in "cats'" and "dogs'" suppress the flagging of these
@@ -350,6 +379,8 @@ class TestRDocMarkupAttributeManager < RDoc::TestCase
                   @am.flow("#fred dogs'"))
 
     assert_equal(["cats' ", crossref("#fred")].flatten, @am.flow("cats' #fred"))
+
+    assert_equal(["(", crossref("#fred"), ")"].flatten, @am.flow("(#fred)"))
   end
 
   def test_tt_html
