@@ -3,18 +3,26 @@ require_relative "helper"
 require "test/unit"
 require "tempfile"
 require "timeout"
+require "open3"
 
 module BasetestReadline
+  RUBY = EnvUtil.rubybin
+
   INPUTRC = "INPUTRC"
+  TERM = "TERM"
   SAVED_ENV = %w[COLUMNS LINES]
 
+  TIMEOUT = 8
+
   def setup
     @saved_env = ENV.values_at(*SAVED_ENV)
     @inputrc, ENV[INPUTRC] = ENV[INPUTRC], IO::NULL
+    @term, ENV[TERM] = ENV[TERM], "vt100"
   end
 
   def teardown
     ENV[INPUTRC] = @inputrc
+    ENV[TERM] = @term
     Readline.instance_variable_set("@completion_proc", nil)
     begin
       Readline.delete_text
@@ -176,7 +184,7 @@ module BasetestReadline
       assert_equal("", line2)
       begin
         assert_equal("", Readline.line_buffer)
-      rescue NotimplementedError
+      rescue NotImplementedError
       end
     end
   end
@@ -449,7 +457,7 @@ module BasetestReadline
         w << "\cr\u3042\u3093"
         w.reopen(IO::NULL)
         assert_equal("\u3046\u3093", Readline.readline("", true), bug6602)
-        Timeout.timeout(2) do
+        Timeout.timeout(TIMEOUT) do
           assert_equal("\u3042\u3093", Readline.readline("", true), bug6602)
         end
         assert_equal(nil,            Readline.readline("", true), bug6602)
@@ -475,6 +483,115 @@ module BasetestReadline
     end
   end
 
+  # TODO Green CI for arm32-linux (Travis CI), and Readline 7.0.
+  def test_interrupt_in_other_thread
+    # Editline and Readline 7.0 can't treat I/O that is not tty.
+    omit "Skip Editline" if /EditLine/n.match(Readline::VERSION)
+    omit "Skip Readline 7.0" if Readline::VERSION == "7.0"
+    omit unless respond_to?(:assert_ruby_status)
+    omit if /mswin|mingw/ =~ RUBY_PLATFORM
+
+    # On 32-bit machine, readline library (or libtinfo) seems to cause SEGV internally even with Readline 8.0
+    # GDB Backtrace: https://gist.github.com/mame/d12b9de3bbc3f16d440c1927398d176a
+    # Maybe the same issue: https://github.com/facebookresearch/nle/issues/120
+    omit if /i[3-6]86-linux/ =~ RUBY_PLATFORM
+
+    if defined?(TestReadline) && self.class == TestReadline
+      use = "use_ext_readline"
+    elsif defined?(TestRelineAsReadline) && self.class == TestRelineAsReadline
+      use = "use_lib_reline"
+    end
+    code = <<-"end;"
+      $stdout.sync = true
+      require 'readline'
+      require 'helper'
+      #{use}
+      puts "Readline::VERSION is \#{Readline::VERSION}."
+      Readline.input = STDIN
+      # 0. Send SIGINT to this script.
+      begin
+        Thread.new{
+          trap(:INT) {
+            puts 'TRAP' # 2. Show 'TRAP' message.
+          }
+          Readline.readline('input> ') # 1. Should keep working and call old trap.
+                                       # 4. Receive "\\n" and return because still working.
+        }.value
+      rescue Interrupt
+        puts 'FAILED' # 3. "Interrupt" shouldn't be raised because trapped.
+        raise
+      end
+      puts 'SUCCEEDED' # 5. Finish correctly.
+    end;
+
+    script = Tempfile.new("interrupt_in_other_thread")
+    script.write code
+    script.close
+
+    log = String.new
+
+    EnvUtil.invoke_ruby(["-I#{__dir__}", script.path], "", true, :merge_to_stdout) do |_in, _out, _, pid|
+      Timeout.timeout(TIMEOUT) do
+        log << "** START **"
+        loop do
+          c = _out.read(1)
+          log << c if c
+          break if log.include?('input> ')
+        end
+        log << "** SIGINT **"
+        sleep 0.5
+        Process.kill(:INT, pid)
+        sleep 0.5
+        loop do
+          c = _out.read(1)
+          log << c if c
+          break if log.include?('TRAP')
+        end
+        begin
+          log << "** NEWLINE **"
+          _in.write "\n"
+        rescue Errno::EPIPE
+          log << "** Errno::EPIPE **"
+          # The "write" will fail if Reline crashed by SIGINT.
+        end
+        interrupt_suppressed = nil
+        loop do
+          c = _out.read(1)
+          log << c if c
+          if log.include?('FAILED')
+            interrupt_suppressed = false
+            break
+          end
+          if log.include?('SUCCEEDED')
+            interrupt_suppressed = true
+            break
+          end
+        end
+        assert interrupt_suppressed, "Should handle SIGINT correctly but raised interrupt.\nLog: #{log}\n----"
+      end
+    rescue Timeout::Error => e
+      Process.kill(:KILL, pid)
+      log << "\nKilled by timeout"
+      assert false, "Timed out to handle SIGINT!\nLog: #{log}\nBacktrace:\n#{e.full_message(highlight: false)}\n----"
+    ensure
+      status = nil
+      begin
+        Timeout.timeout(TIMEOUT) do
+          status = Process.wait2(pid).last
+        end
+      rescue Timeout::Error => e
+        log << "\nKilled by timeout to wait2"
+        Process.kill(:KILL, pid)
+        assert false, "Timed out to wait for terminating a process in a test of SIGINT!\nLog: #{log}\nBacktrace:\n#{e.full_message(highlight: false)}\n----"
+      end
+      assert status&.success?, "Unknown failure with exit status #{status.inspect}\nLog: #{log}\n----"
+    end
+
+    assert log.include?('INT'), "Interrupt was handled correctly."
+  ensure
+    script&.close!
+  end
+
   def test_setting_quoting_detection_proc
     return unless Readline.respond_to?(:quoting_detection_proc=)
 
@@ -492,8 +609,8 @@ module BasetestReadline
     saved_completer_word_break_characters = Readline.completer_word_break_characters
 
     # skip if previous value is nil because Readline... = nil is not allowed.
-    skip unless saved_completer_quote_characters
-    skip unless saved_completer_word_break_characters
+    omit "No completer_quote_characters" unless saved_completer_quote_characters
+    omit "No completer_word_break_characters" unless saved_completer_word_break_characters
 
     return unless Readline.respond_to?(:quoting_detection_proc=)
 
@@ -535,8 +652,8 @@ module BasetestReadline
     saved_completer_word_break_characters = Readline.completer_word_break_characters
 
     # skip if previous value is nil because Readline... = nil is not allowed.
-    skip unless saved_completer_quote_characters
-    skip unless saved_completer_word_break_characters
+    omit "No completer_quote_characters" unless saved_completer_quote_characters
+    omit "No completer_word_break_characters" unless saved_completer_word_break_characters
 
     return unless Readline.respond_to?(:quoting_detection_proc=)
     unless get_default_internal_encoding == Encoding::UTF_8
@@ -715,6 +832,20 @@ module BasetestReadline
     Readline.completer_quote_characters = saved_completer_quote_characters if saved_completer_quote_characters
   end
 
+  def test_without_tty
+    omit "Skip Editline" if /EditLine/n.match(Readline::VERSION)
+    loader = nil
+    if defined?(TestReadline) && self.class == TestReadline
+      loader = "use_ext_readline"
+    elsif defined?(TestRelineAsReadline) && self.class == TestRelineAsReadline
+      loader = "use_lib_reline"
+    end
+    if loader
+      res, exit_status = Open3.capture2e("#{RUBY} -I#{__dir__} -Ilib -rhelper -e '#{loader}; Readline.readline(%{y or n?})'", stdin_data: "y\n")
+      assert exit_status.success?, "It should work fine without tty, but it failed.\nError output:\n#{res}"
+    end
+  end
+
   private
 
   def replace_stdio(stdin_path, stdout_path)
@@ -806,11 +937,16 @@ class TestRelineAsReadline < Test::Unit::TestCase
     super
   end
 
+  def teardown
+    finish_using_lib_reline
+    super
+  end
+
   def get_default_internal_encoding
     if RUBY_PLATFORM =~ /mswin|mingw/
       Encoding.default_internal || Encoding::UTF_8
     else
-      super
+      Reline::IOGate.encoding
     end
   end
 end if defined?(Reline) && ENV["TEST_READLINE_OR_RELINE"] != "Readline"
