@@ -1,5 +1,6 @@
 require_relative 'helper'
-require "reline"
+require 'reline'
+require 'stringio'
 
 class Reline::Test < Reline::TestCase
   class DummyCallbackObject
@@ -65,6 +66,8 @@ class Reline::Test < Reline::TestCase
     Reline.completer_word_break_characters = "[".encode(Encoding::ASCII)
     assert_equal("[", Reline.completer_word_break_characters)
     assert_equal(get_reline_encoding, Reline.completer_word_break_characters.encoding)
+
+    assert_nothing_raised { Reline.completer_word_break_characters = '' }
   ensure
     Reline.completer_word_break_characters = completer_word_break_characters
   end
@@ -89,6 +92,8 @@ class Reline::Test < Reline::TestCase
     Reline.completer_quote_characters = "`".encode(Encoding::ASCII)
     assert_equal("`", Reline.completer_quote_characters)
     assert_equal(get_reline_encoding, Reline.completer_quote_characters.encoding)
+
+    assert_nothing_raised { Reline.completer_quote_characters = '' }
   ensure
     Reline.completer_quote_characters = completer_quote_characters
   end
@@ -132,14 +137,14 @@ class Reline::Test < Reline::TestCase
   end
 
   def test_completion_proc
-    skip unless Reline.completion_proc == nil
+    omit unless Reline.completion_proc == nil
     # Another test can set Reline.completion_proc
 
     # assert_equal(nil, Reline.completion_proc)
 
-    p = proc {}
-    Reline.completion_proc = p
-    assert_equal(p, Reline.completion_proc)
+    dummy_proc = proc {}
+    Reline.completion_proc = dummy_proc
+    assert_equal(dummy_proc, Reline.completion_proc)
 
     l = lambda {}
     Reline.completion_proc = l
@@ -156,9 +161,9 @@ class Reline::Test < Reline::TestCase
   def test_output_modifier_proc
     assert_equal(nil, Reline.output_modifier_proc)
 
-    p = proc {}
-    Reline.output_modifier_proc = p
-    assert_equal(p, Reline.output_modifier_proc)
+    dummy_proc = proc {}
+    Reline.output_modifier_proc = dummy_proc
+    assert_equal(dummy_proc, Reline.output_modifier_proc)
 
     l = lambda {}
     Reline.output_modifier_proc = l
@@ -175,9 +180,9 @@ class Reline::Test < Reline::TestCase
   def test_prompt_proc
     assert_equal(nil, Reline.prompt_proc)
 
-    p = proc {}
-    Reline.prompt_proc = p
-    assert_equal(p, Reline.prompt_proc)
+    dummy_proc = proc {}
+    Reline.prompt_proc = dummy_proc
+    assert_equal(dummy_proc, Reline.prompt_proc)
 
     l = lambda {}
     Reline.prompt_proc = l
@@ -194,9 +199,9 @@ class Reline::Test < Reline::TestCase
   def test_auto_indent_proc
     assert_equal(nil, Reline.auto_indent_proc)
 
-    p = proc {}
-    Reline.auto_indent_proc = p
-    assert_equal(p, Reline.auto_indent_proc)
+    dummy_proc = proc {}
+    Reline.auto_indent_proc = dummy_proc
+    assert_equal(dummy_proc, Reline.auto_indent_proc)
 
     l = lambda {}
     Reline.auto_indent_proc = l
@@ -213,9 +218,9 @@ class Reline::Test < Reline::TestCase
   def test_pre_input_hook
     assert_equal(nil, Reline.pre_input_hook)
 
-    p = proc {}
-    Reline.pre_input_hook = p
-    assert_equal(p, Reline.pre_input_hook)
+    dummy_proc = proc {}
+    Reline.pre_input_hook = dummy_proc
+    assert_equal(dummy_proc, Reline.pre_input_hook)
 
     l = lambda {}
     Reline.pre_input_hook = l
@@ -225,9 +230,9 @@ class Reline::Test < Reline::TestCase
   def test_dig_perfect_match_proc
     assert_equal(nil, Reline.dig_perfect_match_proc)
 
-    p = proc {}
-    Reline.dig_perfect_match_proc = p
-    assert_equal(p, Reline.dig_perfect_match_proc)
+    dummy_proc = proc {}
+    Reline.dig_perfect_match_proc = dummy_proc
+    assert_equal(dummy_proc, Reline.dig_perfect_match_proc)
 
     l = lambda {}
     Reline.dig_perfect_match_proc = l
@@ -242,29 +247,57 @@ class Reline::Test < Reline::TestCase
   end
 
   def test_insert_text
-    # TODO
+    assert_equal('', Reline.line_buffer)
+    assert_equal(0, Reline.point)
+    Reline.insert_text('abc')
+    assert_equal('abc', Reline.line_buffer)
+    assert_equal(3, Reline.point)
   end
 
-  def test_line_buffer
-    # TODO
+  def test_delete_text
+    assert_equal('', Reline.line_buffer)
+    assert_equal(0, Reline.point)
+    Reline.insert_text('abc')
+    assert_equal('abc', Reline.line_buffer)
+    assert_equal(3, Reline.point)
+    Reline.delete_text()
+    assert_equal('', Reline.line_buffer)
+    assert_equal(0, Reline.point)
+    Reline.insert_text('abc')
+    Reline.delete_text(1)
+    assert_equal('a', Reline.line_buffer)
+    assert_equal(1, Reline.point)
+    Reline.insert_text('defghi')
+    Reline.delete_text(2, 2)
+    assert_equal('adghi', Reline.line_buffer)
+    assert_equal(5, Reline.point)
   end
 
-  def test_point
-    # TODO
-  end
-
-  def test_input=
-    # TODO
+  def test_set_input_and_output
     assert_raise(TypeError) do
       Reline.input = "This is not a file."
     end
-  end
-
-  def test_output=
-    # TODO
     assert_raise(TypeError) do
       Reline.output = "This is not a file."
     end
+
+    input, to_write = IO.pipe
+    to_read, output = IO.pipe
+    unless Reline.__send__(:input=, input)
+      omit "Setting to input is not effective on #{Reline::IOGate}"
+    end
+    Reline.output = output
+
+    to_write.write "a\n"
+    result = Reline.readline
+    to_write.close
+    read_text = to_read.read_nonblock(100)
+    assert_equal('a', result)
+    refute(read_text.empty?)
+  ensure
+    input&.close
+    output&.close
+    to_read&.close
   end
 
   def test_vi_editing_mode
@@ -277,8 +310,46 @@ class Reline::Test < Reline::TestCase
     assert_equal(Reline::KeyActor::Emacs, Reline.send(:core).config.editing_mode.class)
   end
 
-  def test_editing_mode
-    # TODO
+  def test_add_dialog_proc
+    dummy_proc = proc {}
+    Reline.add_dialog_proc(:test_proc, dummy_proc)
+    d = Reline.dialog_proc(:test_proc)
+    assert_equal(dummy_proc, d.dialog_proc)
+
+    dummy_proc_2 = proc {}
+    Reline.add_dialog_proc(:test_proc, dummy_proc_2)
+    d = Reline.dialog_proc(:test_proc)
+    assert_equal(dummy_proc_2, d.dialog_proc)
+
+    l = lambda {}
+    Reline.add_dialog_proc(:test_lambda, l)
+    d = Reline.dialog_proc(:test_lambda)
+    assert_equal(l, d.dialog_proc)
+
+    assert_equal(nil, Reline.dialog_proc(:test_nothing))
+
+    assert_raise(ArgumentError) { Reline.add_dialog_proc(:error, 42) }
+    assert_raise(ArgumentError) { Reline.add_dialog_proc(:error, 'hoge') }
+    assert_raise(ArgumentError) { Reline.add_dialog_proc('error', proc {} ) }
+
+    dummy = DummyCallbackObject.new
+    Reline.add_dialog_proc(:dummy, dummy)
+    d = Reline.dialog_proc(:dummy)
+    assert_equal(dummy, d.dialog_proc)
+  end
+
+  def test_add_dialog_proc_with_context
+    dummy_proc = proc {}
+    array = Array.new
+    Reline.add_dialog_proc(:test_proc, dummy_proc, array)
+    d = Reline.dialog_proc(:test_proc)
+    assert_equal(dummy_proc, d.dialog_proc)
+    assert_equal(array, d.context)
+
+    Reline.add_dialog_proc(:test_proc, dummy_proc, nil)
+    d = Reline.dialog_proc(:test_proc)
+    assert_equal(dummy_proc, d.dialog_proc)
+    assert_equal(nil, d.context)
   end
 
   def test_readmultiline
@@ -293,23 +364,23 @@ class Reline::Test < Reline::TestCase
     assert_include(Reline.private_instance_methods, :readline)
   end
 
-  def test_inner_readline
-    # TODO in Reline::Core
-  end
-
   def test_read_io
     # TODO in Reline::Core
   end
 
-  def test_read_escaped_key
-    # TODO in Reline::Core
-  end
-
-  def test_may_req_ambiguous_char_width
-    # TODO in Reline::Core
+  def test_dumb_terminal
+    lib = File.expand_path("../../lib", __dir__)
+    out = IO.popen([{"TERM"=>"dumb"}, "ruby", "-I#{lib}", "-rreline", "-e", "p Reline::IOGate"], &:read)
+    assert_equal("Reline::GeneralIO", out.chomp)
   end
 
   def get_reline_encoding
-    RUBY_PLATFORM =~ /mswin|mingw/ ? Encoding::UTF_8 : Encoding::default_external
+    if encoding = Reline::IOGate.encoding
+      encoding
+    elsif RUBY_PLATFORM =~ /mswin|mingw/
+      Encoding::UTF_8
+    else
+      Encoding::default_external
+    end
   end
 end
