@@ -4,7 +4,7 @@ require 'irb/ruby-lex'
 class TerminationChecker < RubyLex
   def terminated?(code)
     code.gsub!(/\n*$/, '').concat("\n")
-    @tokens = Ripper.lex(code)
+    @tokens = self.class.ripper_lex_without_warning(code)
     continue = process_continue
     code_block_open = check_code_block(code)
     indent = process_nesting_level
@@ -19,8 +19,8 @@ end
 
 class AutoIndent < RubyLex
   def initialize
-    set_input(self)
-    context = Struct.new(:auto_indent_mode, :workspace).new(true, nil)
+    context = Struct.new("MockIRBContext", :auto_indent_mode, :workspace, :local_variables).new(true, nil, [])
+    set_input(self, context: context)
     set_auto_indent(context)
   end
 
