@@ -3,7 +3,36 @@ require 'test/unit'
 require 'resolv'
 require 'socket'
 require 'tempfile'
-require 'minitest/mock'
+
+class Object # :nodoc:
+  def stub name, val_or_callable, &block
+    new_name = "__minitest_stub__#{name}"
+
+    metaclass = class << self; self; end
+
+    if respond_to? name and not methods.map(&:to_s).include? name.to_s then
+      metaclass.send :define_method, name do |*args|
+        super(*args)
+      end
+    end
+
+    metaclass.send :alias_method, new_name, name
+
+    metaclass.send :define_method, name do |*args|
+      if val_or_callable.respond_to? :call then
+        val_or_callable.call(*args)
+      else
+        val_or_callable
+      end
+    end
+
+    yield self
+  ensure
+    metaclass.send :undef_method, name
+    metaclass.send :alias_method, name, new_name
+    metaclass.send :undef_method, new_name
+  end unless method_defined?(:stub) # lib/rubygems/test_case.rb also has the same method definition
+end
 
 class TestResolvDNS < Test::Unit::TestCase
   def setup
@@ -43,7 +72,7 @@ class TestResolvDNS < Test::Unit::TestCase
     begin
       OpenSSL
     rescue LoadError
-      skip 'autoload problem. see [ruby-dev:45021][Bug #5786]'
+      omit 'autoload problem. see [ruby-dev:45021][Bug #5786]'
     end if defined?(OpenSSL)
 
     with_udp('127.0.0.1', 0) {|u|
@@ -132,7 +161,7 @@ class TestResolvDNS < Test::Unit::TestCase
     begin
       OpenSSL
     rescue LoadError
-      skip 'autoload problem. see [ruby-dev:45021][Bug #5786]'
+      omit 'autoload problem. see [ruby-dev:45021][Bug #5786]'
     end if defined?(OpenSSL)
 
     with_udp('127.0.0.1', 0) {|u|
@@ -156,7 +185,7 @@ class TestResolvDNS < Test::Unit::TestCase
             ra =     (flags & 0x0080) >> 7
             z =      (flags & 0x0070) >> 4
             rcode =   flags & 0x000f
-            rest = msg[12..-1]
+            _rest = msg[12..-1]
 
             questions = msg.bytes[12..-1]
             labels = []
@@ -268,6 +297,7 @@ class TestResolvDNS < Test::Unit::TestCase
   end
 
   def test_no_server
+    omit if /mswin/ =~ RUBY_PLATFORM && ENV.key?('GITHUB_ACTIONS') # not working from the beginning
     u = UDPSocket.new
     u.bind("127.0.0.1", 0)
     _, port, _, host = u.addr
@@ -284,7 +314,7 @@ class TestResolvDNS < Test::Unit::TestCase
     rescue Timeout::Error
       if RUBY_PLATFORM.match?(/mingw/)
         # cannot repo locally
-        skip 'Timeout Error on MinGW CI'
+        omit 'Timeout Error on MinGW CI'
       else
         raise Timeout::Error
       end
