@@ -179,6 +179,11 @@ class TestRipper::ScannerEvents < Test::Unit::TestCase
                  scan('backtick', %q[p `make all`])
   end
 
+  def test_colon2_call
+    assert_equal ["::"],
+                 scan('op', %q[ a::b ])
+  end
+
   def test_comma
     assert_equal [','] * 6,
                  scan('comma', %q[ m(0,1,2,3,4,5,6) ])
