@@ -1179,6 +1179,20 @@ Also, a list:
     RUBY_PLATFORM.match("mswin")
   end
 
+  ##
+  # Is this test being run on a version of Ruby built with mingw?
+
+  def self.mingw_windows?
+    RUBY_PLATFORM.match("mingw")
+  end
+
+  ##
+  # Is this test being run on a version of Ruby built with mingw?
+
+  def mingw_windows?
+    RUBY_PLATFORM.match("mingw")
+  end
+
   ##
   # Is this test being run on a ruby/ruby repository?
   #
@@ -1611,7 +1625,7 @@ class Object
       if val_or_callable.respond_to? :call
         val_or_callable.call(*args, &blk)
       else
-        blk.call(*block_args) if blk
+        blk&.call(*block_args)
         val_or_callable
       end
     end
