@@ -1,10 +1,10 @@
 # frozen_string_literal: true
-require 'rubygems/test_case'
-require 'rubygems'
-require 'shellwords'
 
-class TestConfig < Gem::TestCase
+require_relative "helper"
+require "rubygems"
+require "shellwords"
 
+class TestGemConfig < Gem::TestCase
   def test_datadir
     util_make_gems
     spec = Gem::Specification.find_by_name("a")
@@ -13,17 +13,16 @@ class TestConfig < Gem::TestCase
   end
 
   def test_good_rake_path_is_escaped
-    path = Gem::TestCase.class_eval('@@good_rake')
+    path = Gem::TestCase.class_variable_get(:@@good_rake)
     ruby, rake = path.shellsplit
     assert_equal(Gem.ruby, ruby)
     assert_match(/\/good_rake.rb\z/, rake)
   end
 
   def test_bad_rake_path_is_escaped
-    path = Gem::TestCase.class_eval('@@bad_rake')
+    path = Gem::TestCase.class_variable_get(:@@bad_rake)
     ruby, rake = path.shellsplit
     assert_equal(Gem.ruby, ruby)
     assert_match(/\/bad_rake.rb\z/, rake)
   end
-
 end
