@@ -1,7 +1,8 @@
 # frozen_string_literal: true
-require 'rubygems/test_case'
-require 'rubygems/commands/cleanup_command'
-require 'rubygems/installer'
+
+require_relative "helper"
+require "rubygems/commands/cleanup_command"
+require "rubygems/installer"
 
 class TestGemCommandsCleanupCommand < Gem::TestCase
   def setup
@@ -9,8 +10,8 @@ class TestGemCommandsCleanupCommand < Gem::TestCase
 
     @cmd = Gem::Commands::CleanupCommand.new
 
-    @a_1 = util_spec 'a', 1
-    @a_2 = util_spec 'a', 2
+    @a_1 = util_spec "a", 1
+    @a_2 = util_spec "a", 2
 
     install_gem @a_1
     install_gem @a_2
@@ -66,12 +67,12 @@ class TestGemCommandsCleanupCommand < Gem::TestCase
   end
 
   def test_execute_all_dependencies
-    @b_1 = util_spec 'b', 1 do |s|
-      s.add_dependency 'a', '1'
+    @b_1 = util_spec "b", 1 do |s|
+      s.add_dependency "a", "1"
     end
 
-    @b_2 = util_spec 'b', 2 do |s|
-      s.add_dependency 'a', '2'
+    @b_2 = util_spec "b", 2 do |s|
+      s.add_dependency "a", "2"
     end
 
     install_gem @b_1
@@ -86,12 +87,12 @@ class TestGemCommandsCleanupCommand < Gem::TestCase
   end
 
   def test_execute_dev_dependencies
-    @b_1 = util_spec 'b', 1 do |s|
-      s.add_development_dependency 'a', '1'
+    @b_1 = util_spec "b", 1 do |s|
+      s.add_development_dependency "a", "1"
     end
 
-    @c_1 = util_spec 'c', 1 do |s|
-      s.add_development_dependency 'a', '2'
+    @c_1 = util_spec "c", 1 do |s|
+      s.add_development_dependency "a", "2"
     end
 
     install_gem @b_1
@@ -105,12 +106,12 @@ class TestGemCommandsCleanupCommand < Gem::TestCase
   end
 
   def test_execute_without_dev_dependencies
-    @b_1 = util_spec 'b', 1 do |s|
-      s.add_development_dependency 'a', '1'
+    @b_1 = util_spec "b", 1 do |s|
+      s.add_development_dependency "a", "1"
     end
 
-    @c_1 = util_spec 'c', 1 do |s|
-      s.add_development_dependency 'a', '2'
+    @c_1 = util_spec "c", 1 do |s|
+      s.add_development_dependency "a", "2"
     end
 
     install_gem @b_1
@@ -124,14 +125,14 @@ class TestGemCommandsCleanupCommand < Gem::TestCase
   end
 
   def test_execute_all
-    gemhome2 = File.join @tempdir, 'gemhome2'
+    gemhome2 = File.join @tempdir, "gemhome2"
 
     Gem.ensure_gem_subdirectories gemhome2
 
     Gem.use_paths @gemhome, gemhome2
 
-    @b_1 = util_spec 'b', 1
-    @b_2 = util_spec 'b', 2
+    @b_1 = util_spec "b", 1
+    @b_2 = util_spec "b", 2
 
     install_gem @b_1
     install_gem @b_2
@@ -140,15 +141,15 @@ class TestGemCommandsCleanupCommand < Gem::TestCase
 
     @cmd.execute
 
-    assert_equal @gemhome, Gem.dir, 'GEM_HOME'
-    assert_equal [@gemhome, gemhome2], Gem.path.sort, 'GEM_PATH'
+    assert_equal @gemhome, Gem.dir, "GEM_HOME"
+    assert_equal [@gemhome, gemhome2], Gem.path.sort, "GEM_PATH"
 
     assert_path_not_exist @a_1.gem_dir
     assert_path_not_exist @b_1.gem_dir
   end
 
   def test_execute_all_user
-    @a_1_1, = util_gem 'a', '1.1'
+    @a_1_1, = util_gem "a", "1.1"
     @a_1_1 = install_gem @a_1_1 # pick up user install path
 
     Gem::Specification.dirs = [Gem.dir, Gem.user_dir]
@@ -167,7 +168,7 @@ class TestGemCommandsCleanupCommand < Gem::TestCase
   def test_execute_all_user_no_sudo
     FileUtils.chmod 0555, @gemhome
 
-    @a_1_1, = util_gem 'a', '1.1'
+    @a_1_1, = util_gem "a", "1.1"
     @a_1_1 = install_gem @a_1_1, :user_install => true # pick up user install path
 
     Gem::Specification.dirs = [Gem.dir, Gem.user_dir]
@@ -195,11 +196,11 @@ class TestGemCommandsCleanupCommand < Gem::TestCase
   end
 
   def test_execute_keeps_older_versions_with_deps
-    @b_1 = util_spec 'b', 1
-    @b_2 = util_spec 'b', 2
+    @b_1 = util_spec "b", 1
+    @b_2 = util_spec "b", 2
 
-    @c = util_spec 'c', 1 do |s|
-      s.add_dependency 'b', '1'
+    @c = util_spec "c", 1 do |s|
+      s.add_dependency "b", "1"
     end
 
     install_gem @b_1
@@ -216,9 +217,9 @@ class TestGemCommandsCleanupCommand < Gem::TestCase
   def test_execute_ignore_default_gem_verbose
     Gem.configuration.verbose = :really
 
-    @b_1 = util_spec 'b', 1
+    @b_1 = util_spec "b", 1
     @b_default = new_default_spec "b", "2"
-    @b_2 = util_spec 'b', 3
+    @b_2 = util_spec "b", 3
 
     install_gem @b_1
     install_default_gems @b_default
@@ -235,12 +236,12 @@ class TestGemCommandsCleanupCommand < Gem::TestCase
   end
 
   def test_execute_remove_gem_home_only
-    c_1, = util_gem 'c', '1'
-    c_2, = util_gem 'c', '2'
-    d_1, = util_gem 'd', '1'
-    d_2, = util_gem 'd', '2'
-    e_1, = util_gem 'e', '1'
-    e_2, = util_gem 'e', '2'
+    c_1, = util_gem "c", "1"
+    c_2, = util_gem "c", "2"
+    d_1, = util_gem "d", "1"
+    d_2, = util_gem "d", "2"
+    e_1, = util_gem "e", "1"
+    e_2, = util_gem "e", "2"
 
     c_1 = install_gem c_1, :user_install => true # pick up user install path
     c_2 = install_gem c_2
@@ -263,11 +264,11 @@ class TestGemCommandsCleanupCommand < Gem::TestCase
   end
 
   def test_execute_user_install
-    c_1, = util_gem 'c', '1.0'
-    c_2, = util_gem 'c', '1.1'
+    c_1, = util_gem "c", "1.0"
+    c_2, = util_gem "c", "1.1"
 
-    d_1, = util_gem 'd', '1.0'
-    d_2, = util_gem 'd', '1.1'
+    d_1, = util_gem "d", "1.0"
+    d_2, = util_gem "d", "1.1"
 
     c_1 = install_gem c_1, :user_install => true # pick up user install path
     c_2 = install_gem c_2, :user_install => true # pick up user install path
