@@ -1,6 +1,7 @@
 # frozen_string_literal: true
-require_relative 'helper'
-require 'rubygems/commands/stale_command'
+
+require_relative "helper"
+require "rubygems/commands/stale_command"
 
 class TestGemCommandsStaleCommand < Gem::TestCase
   def setup
@@ -11,12 +12,12 @@ class TestGemCommandsStaleCommand < Gem::TestCase
 
   def test_execute_sorts
     files = %w[lib/foo_bar.rb Rakefile]
-    foo_bar = util_spec 'foo_bar' do |gem|
+    foo_bar = util_spec "foo_bar" do |gem|
       gem.files = files
     end
     install_specs foo_bar
 
-    bar_baz = util_spec 'bar_baz' do |gem|
+    bar_baz = util_spec "bar_baz" do |gem|
       gem.files = files
     end
     install_specs bar_baz
