@@ -1,4 +1,5 @@
 # frozen_string_literal: true
+
 require_relative "helper"
 
 require "webrick"
@@ -28,7 +29,7 @@ require "rubygems/package"
 class TestGemRemoteFetcher < Gem::TestCase
   include Gem::DefaultUserInteraction
 
-  SERVER_DATA = <<-EOY.freeze
+  SERVER_DATA = <<-EOY
 --- !ruby/object:Gem::Cache
 gems:
   rake-0.4.11: !ruby/object:Gem::Specification
@@ -1174,10 +1175,10 @@ PeIQQkFng2VVot/WAQbv3ePqWq07g1BBcwIBAg==
   def start_server(data)
     null_logger = NilLog.new
     s = WEBrick::HTTPServer.new(
-      :Port            => 0,
-      :DocumentRoot    => nil,
-      :Logger          => null_logger,
-      :AccessLog       => null_logger
+      :Port => 0,
+      :DocumentRoot => nil,
+      :Logger => null_logger,
+      :AccessLog => null_logger
     )
     s.mount_proc("/kill") {|req, res| s.shutdown }
     s.mount_proc("/yaml") do |req, res|
