@@ -39,11 +39,11 @@ class TestStringIO < Test::Unit::TestCase
   def test_truncate
     io = StringIO.new("")
     io.puts "abc"
-    io.truncate(0)
+    assert_equal(0, io.truncate(0))
     io.puts "def"
     assert_equal("\0\0\0\0def\n", io.string, "[ruby-dev:24190]")
     assert_raise(Errno::EINVAL) { io.truncate(-1) }
-    io.truncate(10)
+    assert_equal(0, io.truncate(10))
     assert_equal("\0\0\0\0def\n\0\0", io.string)
   end
 
@@ -92,11 +92,11 @@ class TestStringIO < Test::Unit::TestCase
     assert_equal("a", StringIO.new("a").gets(chomp: true))
     assert_equal("a", StringIO.new("a\nb").gets(chomp: true))
     assert_equal("abc", StringIO.new("abc\n\ndef\n").gets(chomp: true))
-    assert_equal("abc\n\ndef", StringIO.new("abc\n\ndef\n").gets(nil, chomp: true))
-    assert_equal("abc\n", StringIO.new("abc\n\ndef\n").gets("", chomp: true))
+    assert_equal("abc\n\ndef\n", StringIO.new("abc\n\ndef\n").gets(nil, chomp: true))
+    assert_equal("abc", StringIO.new("abc\n\ndef\n").gets("", chomp: true))
     stringio = StringIO.new("abc\n\ndef\n")
-    assert_equal("abc\n", stringio.gets("", chomp: true))
-    assert_equal("def", stringio.gets("", chomp: true))
+    assert_equal("abc", stringio.gets("", chomp: true))
+    assert_equal("def\n", stringio.gets("", chomp: true))
 
     assert_string("", Encoding::UTF_8, StringIO.new("\n").gets(chomp: true))
 
@@ -111,11 +111,11 @@ class TestStringIO < Test::Unit::TestCase
     assert_equal("a", StringIO.new("a").gets(chomp: true))
     assert_equal("a", StringIO.new("a\r\nb").gets(chomp: true))
     assert_equal("abc", StringIO.new("abc\r\n\r\ndef\r\n").gets(chomp: true))
-    assert_equal("abc\r\n\r\ndef", StringIO.new("abc\r\n\r\ndef\r\n").gets(nil, chomp: true))
-    assert_equal("abc\r\n", StringIO.new("abc\r\n\r\ndef\r\n").gets("", chomp: true))
+    assert_equal("abc\r\n\r\ndef\r\n", StringIO.new("abc\r\n\r\ndef\r\n").gets(nil, chomp: true))
+    assert_equal("abc", StringIO.new("abc\r\n\r\ndef\r\n").gets("", chomp: true))
     stringio = StringIO.new("abc\r\n\r\ndef\r\n")
-    assert_equal("abc\r\n", stringio.gets("", chomp: true))
-    assert_equal("def", stringio.gets("", chomp: true))
+    assert_equal("abc", stringio.gets("", chomp: true))
+    assert_equal("def\r\n", stringio.gets("", chomp: true))
   end
 
   def test_readlines
@@ -258,6 +258,12 @@ class TestStringIO < Test::Unit::TestCase
       f.set_encoding(Encoding::ASCII_8BIT)
     }
     assert_equal("foo\x83".b, f.gets)
+
+    f = StringIO.new()
+    f.set_encoding("ISO-8859-16:ISO-8859-1")
+    assert_equal(Encoding::ISO_8859_16, f.external_encoding)
+    assert_equal(Encoding::ISO_8859_16, f.string.encoding)
+    assert_nil(f.internal_encoding)
   end
 
   def test_mode_error
@@ -586,20 +592,30 @@ class TestStringIO < Test::Unit::TestCase
     end
   end
 
+  def test_each_string_sep
+    f = StringIO.new('a||b||c')
+    assert_equal(["a||", "b||", "c"], f.each("||").to_a)
+    f.rewind
+    assert_equal(["a", "b", "c"], f.each("||", chomp: true).to_a)
+  end
+
   def test_each
     f = StringIO.new("foo\nbar\nbaz\n")
     assert_equal(["foo\n", "bar\n", "baz\n"], f.each.to_a)
     f.rewind
     assert_equal(["foo", "bar", "baz"], f.each(chomp: true).to_a)
-    f = StringIO.new("foo\nbar\n\nbaz\n")
-    assert_equal(["foo\nbar\n\n", "baz\n"], f.each("").to_a)
+    f = StringIO.new("foo\nbar\n\n\nbaz\n")
+    assert_equal(["foo\nbar\n\n\n", "baz\n"], f.each("").to_a)
     f.rewind
-    assert_equal(["foo\nbar\n", "baz"], f.each("", chomp: true).to_a)
+    assert_equal(["foo\nbar", "baz\n"], f.each("", chomp: true).to_a)
 
-    f = StringIO.new("foo\r\nbar\r\n\r\nbaz\r\n")
-    assert_equal(["foo\r\nbar\r\n\r\n", "baz\r\n"], f.each("").to_a)
+    f = StringIO.new("foo\r\nbar\r\n\r\n\r\nbaz\r\n")
+    assert_equal(["foo\r\nbar\r\n\r\n\r\n", "baz\r\n"], f.each("").to_a)
     f.rewind
-    assert_equal(["foo\r\nbar\r\n", "baz"], f.each("", chomp: true).to_a)
+    assert_equal(["foo\r\nbar", "baz\r\n"], f.each("", chomp: true).to_a)
+
+    f = StringIO.new("abc\n\ndef\n")
+    assert_equal(["ab", "c\n", "\nd", "ef", "\n"], f.each(nil, 2, chomp: true).to_a)
   end
 
   def test_putc
@@ -759,6 +775,15 @@ class TestStringIO < Test::Unit::TestCase
     assert_equal("b""\0""a", s.string)
   end
 
+  def test_ungetc_fill
+    count = 100
+    s = StringIO.new
+    s.print 'a' * count
+    s.ungetc('b' * (count * 5))
+    assert_equal((count * 5), s.string.size)
+    assert_match(/\Ab+\z/, s.string)
+  end
+
   def test_ungetbyte_pos
     b = '\\b00010001 \\B00010001 \\b1 \\B1 \\b000100011'
     s = StringIO.new( b )
@@ -784,6 +809,15 @@ class TestStringIO < Test::Unit::TestCase
     assert_equal("b""\0""a", s.string)
   end
 
+  def test_ungetbyte_fill
+    count = 100
+    s = StringIO.new
+    s.print 'a' * count
+    s.ungetbyte('b' * (count * 5))
+    assert_equal((count * 5), s.string.size)
+    assert_match(/\Ab+\z/, s.string)
+  end
+
   def test_frozen
     s = StringIO.new
     s.freeze
@@ -827,18 +861,17 @@ class TestStringIO < Test::Unit::TestCase
   end
 
   def test_overflow
-    omit if RbConfig::SIZEOF["void*"] > RbConfig::SIZEOF["long"]
+    return if RbConfig::SIZEOF["void*"] > RbConfig::SIZEOF["long"]
     limit = RbConfig::LIMITS["INTPTR_MAX"] - 0x10
     assert_separately(%w[-rstringio], "#{<<-"begin;"}\n#{<<-"end;"}")
     begin;
       limit = #{limit}
       ary = []
-      while true
+      begin
         x = "a"*0x100000
         break if [x].pack("p").unpack("i!")[0] < 0
         ary << x
-        omit if ary.size > 100
-      end
+      end while ary.size <= 100
       s = StringIO.new(x)
       s.gets("xxx", limit)
       assert_equal(0x100000, s.pos)
