@@ -5,35 +5,60 @@ require 'tmpdir'
 class TestTmpdir < Test::Unit::TestCase
   def test_tmpdir_modifiable
     tmpdir = Dir.tmpdir
-    assert_equal(false, tmpdir.frozen?)
+    assert_not_predicate(tmpdir, :frozen?)
     tmpdir_org = tmpdir.dup
     tmpdir << "foo"
     assert_equal(tmpdir_org, Dir.tmpdir)
   end
 
   def test_world_writable
-    skip "no meaning on this platform" if /mswin|mingw/ =~ RUBY_PLATFORM
+    omit "no meaning on this platform" if /mswin|mingw/ =~ RUBY_PLATFORM
     Dir.mktmpdir do |tmpdir|
       # ToDo: fix for parallel test
-      olddir, ENV["TMPDIR"] = ENV["TMPDIR"], tmpdir
+      envs = %w[TMPDIR TMP TEMP]
+      oldenv = envs.each_with_object({}) {|v, h| h[v] = ENV.delete(v)}
       begin
-        assert_equal(tmpdir, Dir.tmpdir)
-        File.chmod(0777, tmpdir)
-        assert_not_equal(tmpdir, Dir.tmpdir)
-        newdir = Dir.mktmpdir("d", tmpdir) do |dir|
-          assert_file.directory? dir
-          assert_equal(tmpdir, File.dirname(dir))
-          dir
+        envs.each do |e|
+          tmpdirx = File.join(tmpdir, e)
+          ENV[e] = tmpdirx
+          assert_not_equal(tmpdirx, assert_warn('') {Dir.tmpdir})
+          File.write(tmpdirx, "")
+          assert_not_equal(tmpdirx, assert_warn(/\A#{e} is not a directory/) {Dir.tmpdir})
+          File.unlink(tmpdirx)
+          ENV[e] = tmpdir
+          assert_equal(tmpdir, Dir.tmpdir)
+          File.chmod(0555, tmpdir)
+          assert_not_equal(tmpdir, assert_warn(/\A#{e} is not writable/) {Dir.tmpdir})
+          File.chmod(0777, tmpdir)
+          assert_not_equal(tmpdir, assert_warn(/\A#{e} is world-writable/) {Dir.tmpdir})
+          newdir = Dir.mktmpdir("d", tmpdir) do |dir|
+            assert_file.directory? dir
+            assert_equal(tmpdir, File.dirname(dir))
+            dir
+          end
+          assert_file.not_exist?(newdir)
+          File.chmod(01777, tmpdir)
+          assert_equal(tmpdir, Dir.tmpdir)
+          ENV[e] = nil
         end
-        assert_file.not_exist?(newdir)
-        File.chmod(01777, tmpdir)
-        assert_equal(tmpdir, Dir.tmpdir)
       ensure
-        ENV["TMPDIR"] = olddir
+        ENV.update(oldenv)
       end
     end
   end
 
+  def test_tmpdir_not_empty_parent
+    Dir.mktmpdir do |tmpdir|
+      envs = %w[TMPDIR TMP TEMP]
+      oldenv = envs.each_with_object({}) {|v, h| h[v] = ENV.delete(v)}
+      ENV[envs[0]] = ""
+      ENV[envs[1]] = tmpdir
+      assert_equal(tmpdir, Dir.tmpdir)
+    ensure
+      ENV.update(oldenv)
+    end
+  end
+
   def test_no_homedir
     bug7547 = '[ruby-core:50793]'
     home, ENV["HOME"] = ENV["HOME"], nil
@@ -84,8 +109,6 @@ class TestTmpdir < Test::Unit::TestCase
       target = target.chomp('/') + '/'
       traversal_path = target.sub(/\A\w:/, '') # for DOSISH
       traversal_path = Array.new(target.count('/')-2, '..').join('/') + traversal_path
-      actual = yield traversal_path
-      assert_not_send([File.absolute_path(actual), :start_with?, target])
       [File::SEPARATOR, File::ALT_SEPARATOR].compact.each do |separator|
         actual = yield traversal_path.tr('/', separator)
         assert_not_send([File.absolute_path(actual), :start_with?, target])
