@@ -74,7 +74,7 @@ if defined?(WIN32OLE)
 
     def test_methods
       methods = @dict1.methods
-      assert_include(methods, 'Add')
+      assert_include(methods, :Add)
     end
 
     def test_ole_func_methods
@@ -101,7 +101,7 @@ if defined?(WIN32OLE)
       assert_include(mnames, 'Count')
     end
 
-    def test_ole_mehtod_help
+    def test_ole_method_help
       minfo = @dict1.ole_method_help("Add")
       assert_equal(2, minfo.size_params)
     end
@@ -176,35 +176,6 @@ if defined?(WIN32OLE)
       }
     end
 
-    def test_s_new_exc_svr_tainted
-      th = Thread.start {
-        $SAFE = 1
-        svr = "Scripting.Dictionary"
-        svr.taint
-        Thread.current.report_on_exception = false
-        WIN32OLE.new(svr)
-      }
-      exc = assert_raise(SecurityError) {
-        th.join
-      }
-      assert_match(/insecure object creation - `Scripting.Dictionary'/, exc.message)
-    end
-
-    def test_s_new_exc_host_tainted
-      th = Thread.start {
-        $SAFE = 1
-        svr = "Scripting.Dictionary"
-        host = "localhost"
-        host.taint
-        Thread.current.report_on_exception = false
-        WIN32OLE.new(svr, host)
-      }
-      exc = assert_raise(SecurityError) {
-        th.join
-      }
-      assert_match(/insecure object creation - `localhost'/, exc.message)
-    end
-
     def test_s_new_DCOM
       rshell = WIN32OLE.new("Shell.Application")
       assert_instance_of(WIN32OLE, rshell)
@@ -230,20 +201,6 @@ if defined?(WIN32OLE)
       }
     end
 
-    def test_s_coonect_exc_tainted
-      th = Thread.start {
-        $SAFE = 1
-        svr = "winmgmts:"
-        svr.taint
-        Thread.current.report_on_exception = false
-        WIN32OLE.connect(svr)
-      }
-      exc = assert_raise(SecurityError) {
-        th.join
-      }
-      assert_match(/insecure connection - `winmgmts:'/, exc.message)
-    end
-
     def test_invoke_accept_symbol_hash_key
       fso = WIN32OLE.new('Scripting.FileSystemObject')
       afolder = fso.getFolder(".")
