@@ -1,6 +1,8 @@
 # vcs
 require 'fileutils'
 require 'optparse'
+require 'pp'
+require 'tempfile'
 
 # This library is used by several other tools/ scripts to detect the current
 # VCS in use (e.g. SVN, Git) or to interact with that VCS.
@@ -9,6 +11,22 @@ ENV.delete('PWD')
 
 class VCS
   DEBUG_OUT = STDERR.dup
+
+  def self.dump(obj, pre = nil)
+    out = DEBUG_OUT
+    @pp ||= PP.new(out)
+    @pp.guard_inspect_key do
+      if pre
+        @pp.group(pre.size, pre) {
+          obj.pretty_print(@pp)
+        }
+      else
+        obj.pretty_print(@pp)
+      end
+      @pp.flush
+      out << "\n"
+    end
+  end
 end
 
 unless File.respond_to? :realpath
@@ -19,14 +37,14 @@ unless File.respond_to? :realpath
 end
 
 def IO.pread(*args)
-  VCS::DEBUG_OUT.puts(args.inspect) if $DEBUG
+  VCS.dump(args, "args: ") if $DEBUG
   popen(*args) {|f|f.read}
 end
 
 module DebugPOpen
   refine IO.singleton_class do
     def popen(*args)
-      VCS::DEBUG_OUT.puts args.inspect if $DEBUG
+      VCS.dump(args, "args: ") if $DEBUG
       super
     end
   end
@@ -34,7 +52,7 @@ end
 using DebugPOpen
 module DebugSystem
   def system(*args)
-    VCS::DEBUG_OUT.puts args.inspect if $DEBUG
+    VCS.dump(args, "args: ") if $DEBUG
     exception = false
     opts = Hash.try_convert(args[-1])
     if RUBY_VERSION >= "2.6"
@@ -52,9 +70,6 @@ module DebugSystem
     ret
   end
 end
-module Kernel
-  prepend(DebugSystem)
-end
 
 class VCS
   prepend(DebugSystem) if defined?(DebugSystem)
@@ -65,12 +80,16 @@ class VCS
     @@dirs << [dir, self, pred]
   end
 
-  def self.detect(path = '.', options = {}, parser = nil)
+  def self.detect(path = '.', options = {}, parser = nil, **opts)
+    options.update(opts)
     uplevel_limit = options.fetch(:uplevel_limit, 0)
     curr = path
     begin
       @@dirs.each do |dir, klass, pred|
         if pred ? pred[curr, dir] : File.directory?(File.join(curr, dir))
+          if klass.const_defined?(:COMMAND)
+            IO.pread([{'LANG' => 'C', 'LC_ALL' => 'C'}, klass::COMMAND, "--version"]) rescue next
+          end
           vcs = klass.new(curr)
           vcs.define_options(parser) if parser
           vcs.set_options(options)
@@ -94,9 +113,23 @@ class VCS
     parser.separator("  VCS common options:")
     parser.define("--[no-]dryrun") {|v| opts[:dryrun] = v}
     parser.define("--[no-]debug") {|v| opts[:debug] = v}
+    parser.define("-z", "--zone=OFFSET", /\A[-+]\d\d:\d\d\z/) {|v| opts[:zone] = v}
     opts
   end
 
+  def release_date(time)
+    t = time.getlocal(@zone)
+    [
+      t.strftime('#define RUBY_RELEASE_YEAR %Y'),
+      t.strftime('#define RUBY_RELEASE_MONTH %-m'),
+      t.strftime('#define RUBY_RELEASE_DAY %-d'),
+    ]
+  end
+
+  def self.short_revision(rev)
+    rev
+  end
+
   attr_reader :srcdir
 
   def initialize(path)
@@ -114,6 +147,7 @@ class VCS
   def set_options(opts)
     @debug = opts.fetch(:debug) {$DEBUG}
     @dryrun = opts.fetch(:dryrun) {@debug}
+    @zone = opts.fetch(:zone) {'+09:00'}
   end
 
   attr_reader :dryrun, :debug
@@ -134,7 +168,7 @@ class VCS
     end
     last, changed, modified, *rest = (
       begin
-        if NullDevice
+        if NullDevice and !debug?
           save_stderr = STDERR.dup
           STDERR.reopen NullDevice, 'w'
         end
@@ -206,6 +240,36 @@ class VCS
     revision_handler(rev).short_revision(rev)
   end
 
+  # make-snapshot generates only release_date whereas file2lastrev generates both release_date and release_datetime
+  def revision_header(last, release_date, release_datetime = nil, branch = nil, title = nil, limit: 20)
+    short = short_revision(last)
+    if /[^\x00-\x7f]/ =~ title and title.respond_to?(:force_encoding)
+      title = title.dup.force_encoding("US-ASCII")
+    end
+    code = [
+      "#define RUBY_REVISION #{short.inspect}",
+    ]
+    unless short == last
+      code << "#define RUBY_FULL_REVISION #{last.inspect}"
+    end
+    if branch
+      e = '..'
+      name = branch.sub(/\A(.{#{limit-e.size}}).{#{e.size+1},}/o) {$1+e}
+      name = name.dump.sub(/\\#/, '#')
+      code << "#define RUBY_BRANCH_NAME #{name}"
+    end
+    if title
+      title = title.dump.sub(/\\#/, '#')
+      code << "#define RUBY_LAST_COMMIT_TITLE #{title}"
+    end
+    if release_datetime
+      t = release_datetime.utc
+      code << t.strftime('#define RUBY_RELEASE_DATETIME "%FT%TZ"')
+    end
+    code += self.release_date(release_date)
+    code
+  end
+
   class SVN < self
     register(".svn")
     COMMAND = ENV['SVN'] || 'svn'
@@ -214,10 +278,6 @@ class VCS
       "r#{rev}"
     end
 
-    def self.short_revision(rev)
-      rev
-    end
-
     def _get_revisions(path, srcdir = nil)
       if srcdir and self.class.local_path?(path)
         path = File.join(srcdir, path)
@@ -341,7 +401,7 @@ class VCS
       rev.to_i if rev
     end
 
-    def export_changelog(url, from, to, path)
+    def export_changelog(url = '.', from = nil, to = nil, _path = nil, path: _path)
       range = [to || 'HEAD', (from ? from+1 : branch_beginning(url))].compact.join(':')
       IO.popen({'TZ' => 'JST-9', 'LANG' => 'C', 'LC_ALL' => 'C'},
                %W"#{COMMAND} log -r#{range} #{url}") do |r|
@@ -352,7 +412,7 @@ class VCS
     def commit
       args = %W"#{COMMAND} commit"
       if dryrun?
-        VCS::DEBUG_OUT.puts(args.inspect)
+        VCS.dump(args, "commit: ")
         return true
       end
       system(*args)
@@ -360,8 +420,21 @@ class VCS
   end
 
   class GIT < self
-    register(".git") {|path, dir| File.exist?(File.join(path, dir))}
-    COMMAND = ENV["GIT"] || 'git'
+    register(".git") do |path, dir|
+      SAFE_DIRECTORIES ||=
+        begin
+          command = ENV["GIT"] || 'git'
+          dirs = IO.popen(%W"#{command} config --global --get-all safe.directory", &:read).split("\n")
+        rescue
+          command = nil
+          dirs = []
+        ensure
+          VCS.dump(dirs, "safe.directory: ") if $DEBUG
+          COMMAND = command
+        end
+
+      COMMAND and File.exist?(File.join(path, dir))
+    end
 
     def cmd_args(cmds, srcdir = nil)
       (opts = cmds.last).kind_of?(Hash) or cmds << (opts = {})
@@ -369,7 +442,7 @@ class VCS
       if srcdir
         opts[:chdir] ||= srcdir
       end
-      VCS::DEBUG_OUT.puts cmds.inspect if debug?
+      VCS.dump(cmds, "cmds: ") if debug? and !$DEBUG
       cmds
     end
 
@@ -379,7 +452,7 @@ class VCS
 
     def cmd_read_at(srcdir, cmds)
       result = without_gitconfig { IO.pread(*cmd_args(cmds, srcdir)) }
-      VCS::DEBUG_OUT.puts result.inspect if debug?
+      VCS.dump(result, "result: ") if debug?
       result
     end
 
@@ -400,7 +473,14 @@ class VCS
     def _get_revisions(path, srcdir = nil)
       ref = Branch === path ? path.to_str : 'HEAD'
       gitcmd = [COMMAND]
-      last = cmd_read_at(srcdir, [[*gitcmd, 'rev-parse', ref]]).rstrip
+      last = nil
+      IO.pipe do |r, w|
+        last = cmd_read_at(srcdir, [[*gitcmd, 'rev-parse', ref, err: w]]).rstrip
+        w.close
+        unless r.eof?
+          raise "#{COMMAND} rev-parse failed\n#{r.read.gsub(/^(?=\s*\S)/, '  ')}"
+        end
+      end
       log = cmd_read_at(srcdir, [[*gitcmd, 'log', '-n1', '--date=iso', '--pretty=fuller', *path]])
       changed = log[/\Acommit (\h+)/, 1]
       modified = log[/^CommitDate:\s+(.*)/, 1]
@@ -462,16 +542,35 @@ class VCS
     end
 
     def without_gitconfig
-      home = ENV.delete('HOME')
+      envs = (%w'HOME XDG_CONFIG_HOME' + ENV.keys.grep(/\AGIT_/)).each_with_object({}) do |v, h|
+        h[v] = ENV.delete(v)
+      end
+      ENV['GIT_CONFIG_SYSTEM'] = NullDevice
+      ENV['GIT_CONFIG_GLOBAL'] = global_config
       yield
     ensure
-      ENV['HOME'] = home if home
+      ENV.update(envs)
+    end
+
+    def global_config
+      return NullDevice if SAFE_DIRECTORIES.empty?
+      unless @gitconfig
+        @gitconfig = Tempfile.new(%w"vcs_ .gitconfig")
+        @gitconfig.close
+        ENV['GIT_CONFIG_GLOBAL'] = @gitconfig.path
+        SAFE_DIRECTORIES.each do |dir|
+          system(*%W[#{COMMAND} config --global --add safe.directory #{dir}])
+        end
+        VCS.dump(`#{COMMAND} config --global --get-all safe.directory`, "safe.directory: ") if debug?
+      end
+      @gitconfig.path
     end
 
     def initialize(*)
       super
       @srcdir = File.realpath(@srcdir)
-      VCS::DEBUG_OUT.puts @srcdir.inspect if debug?
+      @gitconfig = nil
+      VCS.dump(@srcdir, "srcdir: ") if debug?
       self
     end
 
@@ -521,11 +620,11 @@ class VCS
 
     def branch_beginning(url)
       cmd_read(%W[ #{COMMAND} log -n1 --format=format:%H
-                   --author=matz --committer=matz --grep=has\ started
+                   --author=matz --committer=matz --grep=started\\.$
                    #{url.to_str} -- version.h include/ruby/version.h])
     end
 
-    def export_changelog(url, from, to, path)
+    def export_changelog(url = '@', from = nil, to = nil, _path = nil, path: _path, base_url: nil)
       svn = nil
       from, to = [from, to].map do |rev|
         rev or next
@@ -541,51 +640,75 @@ class VCS
         warn "no starting commit found", uplevel: 1
         from = nil
       end
-      unless svn or system(*%W"#{COMMAND} fetch origin refs/notes/commits:refs/notes/commits",
+      if svn or system(*%W"#{COMMAND} fetch origin refs/notes/commits:refs/notes/commits",
                            chdir: @srcdir, exception: false)
-        abort "Could not fetch notes/commits tree"
-      end
-      system(*%W"#{COMMAND} fetch origin refs/notes/log-fix:refs/notes/log-fix",
+        system(*%W"#{COMMAND} fetch origin refs/notes/log-fix:refs/notes/log-fix",
                chdir: @srcdir, exception: false)
+      else
+        warn "Could not fetch notes/commits tree", uplevel: 1
+      end
       to ||= url.to_str
       if from
         arg = ["#{from}^..#{to}"]
       else
         arg = ["--since=25 Dec 00:00:00", to]
       end
-      if svn
-        format_changelog_as_svn(path, arg)
+      writer =
+        if svn
+          format_changelog_as_svn(path, arg)
+        else
+          if base_url == true
+            remote, = upstream
+            if remote &&= cmd_read(env, %W[#{COMMAND} remote get-url --no-push #{remote}])
+              remote.chomp!
+              # hack to redirect git.r-l.o to github
+              remote.sub!(/\Agit@git\.ruby-lang\.org:/, 'git@github.com:ruby/')
+              remote.sub!(/\Agit@(.*?):(.*?)(?:\.git)?\z/, 'https://\1/\2/commit/')
+            end
+            base_url = remote
+          end
+          format_changelog(path, arg, base_url)
+        end
+      if !path or path == '-'
+        writer[$stdout]
       else
-        format_changelog(path, arg)
+        File.open(path, 'wb', &writer)
       end
     end
 
-    def format_changelog(path, arg)
+    LOG_FIX_REGEXP_SEPARATORS = '/!:;|,#%&'
+
+    def format_changelog(path, arg, base_url = nil)
       env = {'TZ' => 'JST-9', 'LANG' => 'C', 'LC_ALL' => 'C'}
-      cmd = %W"#{COMMAND} log --format=medium --notes=commits --notes=log-fix --topo-order --no-merges"
+      cmd = %W"#{COMMAND} log --format=fuller --notes=commits --notes=log-fix --topo-order --no-merges"
       date = "--date=iso-local"
-      unless system(env, *cmd, date, chdir: @srcdir, out: NullDevice, exception: false)
+      unless system(env, *cmd, date, "-1", chdir: @srcdir, out: NullDevice, exception: false)
         date = "--date=iso"
       end
       cmd << date
       cmd.concat(arg)
-      File.open(path, 'w') do |w|
+      proc do |w|
         w.print "-*- coding: utf-8 -*-\n\n"
+        w.print "base-url = #{base_url}\n\n" if base_url
         cmd_pipe(env, cmd, chdir: @srcdir) do |r|
           while s = r.gets("\ncommit ")
             h, s = s.split(/^$/, 2)
-            h.gsub!(/^(?:Author|Date): /, '  \&')
+            h.gsub!(/^(?:(?:Author|Commit)(?:Date)?|Date): /, '  \&')
             if s.sub!(/\nNotes \(log-fix\):\n((?: +.*\n)+)/, '')
               fix = $1
               s = s.lines
               fix.each_line do |x|
-                if %r[^ +(\d+)s/(.+)/(.*)/] =~ x
+                case x
+                when %r[^ +(\d+)s([#{LOG_FIX_REGEXP_SEPARATORS}])(.+)\2(.*)\2]o
+                  n = $1.to_i
+                  wrong = $3
+                  correct = $4
                   begin
-                    s[$1.to_i][$2] = $3
+                    s[n][wrong] = correct
                   rescue IndexError
-                    message = ["format_changelog failed to replace #{$2.dump} with #{$3.dump} at #$1\n"]
-                    from = [1, $1.to_i-2].max
-                    to = [s.size-1, $1.to_i+2].min
+                    message = ["format_changelog failed to replace #{wrong.dump} with #{correct.dump} at #$1\n"]
+                    from = [1, n-2].max
+                    to = [s.size-1, n+2].min
                     s.each_with_index do |e, i|
                       next if i < from
                       break if to < i
@@ -593,10 +716,22 @@ class VCS
                     end
                     raise message.join('')
                   end
+                when %r[^( +)(\d+)i([#{LOG_FIX_REGEXP_SEPARATORS}])(.*)\3]o
+                  s[$2.to_i, 0] = "#{$1}#{$4}\n"
+                when %r[^ +(\d+)(?:,(\d+))?d]
+                  n = $1.to_i
+                  e = $2
+                  s[n..(e ? e.to_i : n)] = []
                 end
               end
               s = s.join('')
             end
+
+            if %r[^ +(https://github\.com/[^/]+/[^/]+/)commit/\h+\n(?=(?: +\n(?i: +Co-authored-by: .*\n)+)?(?:\n|\Z))] =~ s
+              issue = "#{$1}pull/"
+              s.gsub!(/\b(?:(?i:fix(?:e[sd])?) +|GH-)\K#(?=\d+\b)|\(\K#(?=\d+\))/) {issue}
+            end
+
             s.gsub!(/ +\n/, "\n")
             s.sub!(/^Notes:/, '  \&')
             w.print h, s
@@ -608,7 +743,7 @@ class VCS
     def format_changelog_as_svn(path, arg)
       cmd = %W"#{COMMAND} log --topo-order --no-notes -z --format=%an%n%at%n%B"
       cmd.concat(arg)
-      File.open(path, 'w') do |w|
+      proc do |w|
         sep = "-"*72 + "\n"
         w.print sep
         cmd_pipe(cmd) do |r|
@@ -640,13 +775,13 @@ class VCS
 
     def commit(opts = {})
       args = [COMMAND, "push"]
-      args << "-n" if dryrun
+      args << "-n" if dryrun?
       remote, branch = upstream
       args << remote
       branches = %W[refs/notes/commits:refs/notes/commits HEAD:#{branch}]
       if dryrun?
         branches.each do |b|
-          VCS::DEBUG_OUT.puts((args + [b]).inspect)
+          VCS.dump(args + [b], "commit: ")
         end
         return true
       end
@@ -676,7 +811,7 @@ class VCS
       commits.each_with_index do |l, i|
         r, a, c = l.split(' ')
         dcommit = [COMMAND, "svn", "dcommit"]
-        dcommit.insert(-2, "-n") if dryrun
+        dcommit.insert(-2, "-n") if dryrun?
         dcommit << "--add-author-from" unless a == c
         dcommit << r
         system(*dcommit) or return false
@@ -696,4 +831,15 @@ class VCS
       true
     end
   end
+
+  class Null < self
+    def get_revisions(path, srcdir = nil)
+      @modified ||= Time.now - 10
+      return nil, nil, @modified
+    end
+
+    def revision_header(last, release_date, release_datetime = nil, branch = nil, title = nil, limit: 20)
+      self.release_date(release_date)
+    end
+  end
 end
