@@ -15,6 +15,37 @@
 static void vm_analysis_insn(int insn);
 #endif
 
+MAYBE_UNUSED(static void vm_insns_counter_count_insn(int insn));
+#if USE_INSNS_COUNTER
+static size_t rb_insns_counter[VM_INSTRUCTION_SIZE];
+
+static void
+vm_insns_counter_count_insn(int insn)
+{
+    rb_insns_counter[insn]++;
+}
+
+__attribute__((destructor))
+static void
+vm_insns_counter_show_results_at_exit(void)
+{
+    int insn_end = (ruby_vm_event_enabled_global_flags & ISEQ_TRACE_EVENTS)
+        ? VM_INSTRUCTION_SIZE : VM_INSTRUCTION_SIZE / 2;
+
+    size_t total = 0;
+    for (int insn = 0; insn < insn_end; insn++)
+        total += rb_insns_counter[insn];
+
+    for (int insn = 0; insn < insn_end; insn++) {
+        fprintf(stderr, "[RUBY_INSNS_COUNTER]\t%-32s%'12"PRIuSIZE" (%4.1f%%)\n",
+                insn_name(insn), rb_insns_counter[insn],
+                100.0 * rb_insns_counter[insn] / total);
+    }
+}
+#else
+static void vm_insns_counter_count_insn(int insn) {}
+#endif
+
 #if VMDEBUG > 0
 #define DECL_SC_REG(type, r, reg) register type reg_##r
 
@@ -24,7 +55,7 @@ static void vm_analysis_insn(int insn);
 #elif defined(__GNUC__) && defined(__i386__)
 #define DECL_SC_REG(type, r, reg) register type reg_##r __asm__("e" reg)
 
-#elif defined(__GNUC__) && defined(__powerpc64__)
+#elif defined(__GNUC__) && (defined(__powerpc64__) || defined(__POWERPC__))
 #define DECL_SC_REG(type, r, reg) register type reg_##r __asm__("r" reg)
 
 #elif defined(__GNUC__) && defined(__aarch64__)
@@ -35,17 +66,6 @@ static void vm_analysis_insn(int insn);
 #endif
 /* #define DECL_SC_REG(r, reg) VALUE reg_##r */
 
-#if VM_DEBUG_STACKOVERFLOW
-NORETURN(static void vm_stack_overflow_for_insn(void));
-static void
-vm_stack_overflow_for_insn(void)
-{
-    rb_bug("CHECK_VM_STACK_OVERFLOW_FOR_INSN: should not overflow here. "
-	   "Please contact ruby-core/dev with your (a part of) script. "
-	   "This check will be removed soon.");
-}
-#endif
-
 #if !OPT_CALL_THREADED_CODE
 static VALUE
 vm_exec_core(rb_execution_context_t *ec, VALUE initial)
@@ -72,7 +92,7 @@ vm_exec_core(rb_execution_context_t *ec, VALUE initial)
     DECL_SC_REG(rb_control_frame_t *, cfp, "15");
 #define USE_MACHINE_REGS 1
 
-#elif defined(__GNUC__) && defined(__powerpc64__)
+#elif defined(__GNUC__) && (defined(__powerpc64__) || defined(__POWERPC__))
     DECL_SC_REG(const VALUE *, pc, "14");
     DECL_SC_REG(rb_control_frame_t *, cfp, "15");
 #define USE_MACHINE_REGS 1
@@ -85,6 +105,8 @@ vm_exec_core(rb_execution_context_t *ec, VALUE initial)
 #else
     register rb_control_frame_t *reg_cfp;
     const VALUE *reg_pc;
+#define USE_MACHINE_REGS 0
+
 #endif
 
 #if USE_MACHINE_REGS
@@ -107,7 +129,7 @@ vm_exec_core(rb_execution_context_t *ec, VALUE initial)
 #if OPT_TOKEN_THREADED_CODE || OPT_DIRECT_THREADED_CODE
 #include "vmtc.inc"
     if (UNLIKELY(ec == 0)) {
-	return (VALUE)insns_address_table;
+        return (VALUE)insns_address_table;
     }
 #endif
     reg_cfp = ec->cfp;
@@ -154,22 +176,22 @@ vm_exec_core(rb_execution_context_t *ec, VALUE initial)
     rb_thread_t *th;
 
     while (1) {
-	reg_cfp = ((rb_insn_func_t) (*GET_PC()))(ec, reg_cfp);
+        reg_cfp = ((rb_insn_func_t) (*GET_PC()))(ec, reg_cfp);
 
-	if (UNLIKELY(reg_cfp == 0)) {
-	    break;
-	}
+        if (UNLIKELY(reg_cfp == 0)) {
+            break;
+        }
     }
 
-    if ((th = rb_ec_thread_ptr(ec))->retval != Qundef) {
-	VALUE ret = th->retval;
-	th->retval = Qundef;
-	return ret;
+    if (!UNDEF_P((th = rb_ec_thread_ptr(ec))->retval)) {
+        VALUE ret = th->retval;
+        th->retval = Qundef;
+        return ret;
     }
     else {
-	VALUE err = ec->errinfo;
-	ec->errinfo = Qnil;
-	return err;
+        VALUE err = ec->errinfo;
+        ec->errinfo = Qnil;
+        return err;
     }
 }
 #endif
